# -*- coding: utf-8 -*-
# Copyright (C) 2018-2021 by the Free Software Foundation, Inc.
#
# This file is part of Postorius.
#
# Postorius is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option)
# any later version.
#
# Postorius is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along with
# Postorius.  If not, see <http://www.gnu.org/licenses/>.
#

import urllib

from django.conf import settings
from django.contrib.auth.models import User
from django.contrib.sites.models import Site
from django.urls import reverse

from allauth.account.models import EmailAddress
from django_mailman3.models import MailDomain

from postorius.models import TEMPLATES_LIST, EmailTemplate
from postorius.tests.utils import ViewTestCase


class DomainTemplateViewTest(ViewTestCase):

    def setUp(self):
        super().setUp()
        self.domain = self.mm_client.create_domain('example.com')
        self.su = User.objects.create_superuser(
            username='su', email='su@example.com', password='testpass')
        self.domain_owner = User.objects.create_user(
            username='domain_owner', email='do@example.com',
            password='testpass')
        self.testuser = User.objects.create_user(
            username='testuser', email='testuser@example.com',
            password='testpass')
        for user in (self.testuser, self.su, self.domain_owner):
            EmailAddress.objects.create(
                user=user, email=user.email, verified=True)
        MailDomain.objects.create(
            site=Site.objects.get_current(), mail_domain='example.com')
        self.domain.add_owner('do@example.com')

    def tearDown(self):
        EmailTemplate.objects.all().delete()
        # We do this because instead of returning an empty list, we get
        # a 404 error. This must be fixed in Client.

        super().tearDown()

    def test_access_anonymous(self):
        self.assertEqual(403, self._check_access(None, reverse(
            'domain_template_list', args=('example.com',))))
        self.assertEqual(403, self._check_access(None, reverse(
            'domain_template_new', args=('example.com',))))
        self.assertEqual(403, self._check_access(None, reverse(
            'domain_template_update', args=('example.com', 'template_id'))))
        self.assertEqual(403, self._check_access(None, reverse(
            'domain_template_delete', args=('example.com', 'template_id'))))

    def _check_access(self, user, url):
        if user is not None:
            self.client.login(
                username=user.username, password=user.password)
        response = self.client.get(url)
        return response.status_code

    # Index Checks that access control works as expected. We don't test for
    # access control in other Template Views.
    def test_template_index_view_non_owner(self):
        # Test that the templates are accessible, but only to superuser and
        # domain owners.
        url = reverse('domain_template_list', args=('example.com',))
        response = self.client.login(username='testuser', password='testpass')
        self.assertTrue(response)
        response = self.client.get(url)
        self.assertEqual(response.status_code, 403)

    def test_template_index_view_owner(self):
        # Not let's try domain owner.
        url = reverse('domain_template_list', args=('example.com',))
        response = self.client.login(
            username='domain_owner', password='testpass')
        response = self.client.get(url)
        self.assertEqual(response.status_code, 200)

    def test_template_index_view_superuser(self):
        url = reverse('domain_template_list', args=('example.com',))
        # Now, let's try the superuser.
        response = self.client.login(username='su', password='testpass')
        self.assertTrue(response)
        response = self.client.get(url)
        self.assertEqual(response.status_code, 200)

    def test_template_create_view(self):
        self.client.login(
            username='domain_owner', password='testpass')
        # First, let's make sure we can GET the view.
        url = reverse('domain_template_new', args=('example.com',))
        response = self.client.get(url)
        self.assertEqual(response.status_code, 200)
        self.assertTrue('New Template' in str(response.content))
        # Now, let's try to create a new Template.
        data = {'name': 'list:admin:action:post',
                'data': 'This is test data.'}
        response = self.client.post(url, data=data)
        self.assertEqual(response.status_code, 302)
        self.assertEqual(
            response.url,
            reverse('domain_template_list', args=('example.com',)))
        self.assertTrue(EmailTemplate.objects.filter(
            context='domain',
            name='list:admin:action:post',
            identifier='example.com').exists())

    def test_template_create_view_whitespace(self):
        self.client.login(
            username='domain_owner', password='testpass')
        # First, let's make sure we can GET the view.
        url = reverse('domain_template_new', args=('example.com',))
        response = self.client.get(url)
        self.assertEqual(response.status_code, 200)
        self.assertTrue('New Template' in str(response.content))
        # Now, let's try to create a new Template with leading whitespace.
        data = {'name': 'list:admin:action:post',
                'data': '\n\n\t\tThis is test data.'}
        response = self.client.post(url, data=data)
        self.assertEqual(response.status_code, 302)
        # Now, let's check to make sure that the data wasn't stripped.
        templates = EmailTemplate.objects.get(
            name='list:admin:action:post', identifier='example.com')
        self.assertEqual(templates.data, '\n\n\t\tThis is test data.')

    def test_template_create_all(self):
        # Test that we can create all the template options.
        self.client.login(
            username='domain_owner', password='testpass')
        url = reverse('domain_template_new', args=('example.com',))
        for template_name, template_desc in TEMPLATES_LIST:
            # XXX(maxking): This is a bit weird, but Core for some reason
            # causes problems with this particular template. I am going to try
            # to debug this in Core.
            if template_name == 'list:user:notice:welcome':
                continue
            data = {'name': template_name,
                    'data': 'This is test data.'}
            response = self.client.post(url, data=data)
            self.assertEqual(response.status_code, 302, template_name)

    def test_template_unique_property(self):
        # Test that we can create templates with uniqueness.
        self.client.login(username='domain_owner', password='testpass')
        url = reverse('domain_template_new', args=('example.com',))
        data = {'name': 'list:admin:action:post',
                'data': 'This is test data.'}
        response = self.client.post(url, data=data)
        self.assertEqual(response.status_code, 302)
        template = EmailTemplate.objects.get(
            identifier='example.com', context='domain', name=data['name'])
        self.assertEqual(template.data, 'This is test data.')
        # Now, let's create a 2nd domain, and set a template with different
        # domain and same name.
        dom2 = self.mm_client.create_domain('example.net')
        url = reverse('domain_template_new', args=('example.net',))
        # Let's become superuser to post to this.
        self.assertTrue(self.client.login(username='su', password='testpass'))
        response = self.client.post(url, data=data)
        self.assertEqual(response.status_code, 302)
        templates = EmailTemplate.objects.filter(name=data['name'])
        self.assertEqual(len(templates), 2)
        # clean up.
        try:
            for template in dom2.templates:
                template.delete()
        except urllib.error.HTTPError:
            pass

    def test_template_delete_view(self):
        self.client.login(
            username='domain_owner', password='testpass')
        template = EmailTemplate.objects.create(
            name='list:admin:action:post',
            data='This is some new data.',
            context='domain',
            identifier='example.com')
        # First, make sure we get the confirmation page when we GET.
        url = reverse('domain_template_delete',
                      args=('example.com', template.id))
        response = self.client.get(url)
        self.assertEqual(response.status_code, 200)
        self.assertTrue('Delete Template'
                        in str(response.content))
        # Now, let's try to Delete this domain.
        response = self.client.post(url)
        self.assertEqual(response.status_code, 302)
        self.assertEqual(
            response.url,
            reverse('domain_template_list', args=('example.com',)))
        # Make sure that this domain is now deleted.
        self.assertFalse(EmailTemplate.objects.filter(
            context='domain',
            name='list:admin:action:post',
            identifier='example.com').exists())

    def test_template_update_view(self):
        self.client.login(
            username='domain_owner', password='testpass')
        data = dict(name='list:admin:action:post',
                    context='domain',
                    identifier='example.com',
                    data='This is some new data.')
        template = EmailTemplate.objects.create(**data)
        # First, make sure we can GET the page.
        url = reverse('domain_template_update',
                      args=('example.com', template.id))
        response = self.client.get(url)
        self.assertEqual(response.status_code, 200)
        self.assertTrue('Edit Template'
                        in str(response.content))
        # Now, let's try to update this template by POST'ing.
        data = {'data': 'This is the update template data.'}
        response = self.client.post(url, data)
        self.assertEqual(response.status_code, 302)
        template = EmailTemplate.objects.get(pk=template.id)
        self.assertEqual(template.data, data['data'])


class MailingListTemplateViewTest(ViewTestCase):

    def setUp(self):
        super().setUp()
        self.domain = self.mm_client.create_domain('example.com')
        self.su = User.objects.create_superuser(
            username='su', email='su@example.com', password='testpass')
        self.list_owner = User.objects.create_user(
            username='list_owner', email='owner@example.com',
            password='testpass')
        self.list_mod = User.objects.create_user(
            username='list_mod', email='mod@example.com',
            password='testpass')
        self.testuser = User.objects.create_user(
            username='testuser', email='testuser@example.com',
            password='testpass')
        for user in (self.testuser, self.su, self.list_owner, self.list_mod):
            EmailAddress.objects.create(
                user=user, email=user.email, verified=True)
        MailDomain.objects.create(
            site=Site.objects.get_current(), mail_domain='example.com')
        self.mlist = self.domain.create_list('fun')
        self.mlist.add_owner('owner@example.com')
        self.mlist.add_moderator('mod@example.com')

    def tearDown(self):
        for each_level in (
                self.mm_client.templates,
                self.domain.templates,
                self.mlist.templates
        ):
            try:
                for template in each_level:
                    template.delete()
            except urllib.error.HTTPError:
                pass

        super().tearDown()

    def test_access_anonymous(self):
        self.assertEqual(403, self._check_access(None, reverse(
            'list_template_list', args=('fun.example.com',))))
        self.assertEqual(403, self._check_access(None, reverse(
            'list_template_new', args=('fun.example.com',))))
        self.assertEqual(403, self._check_access(None, reverse(
            'list_template_update', args=('fun.example.com', 'template_id'))))
        self.assertEqual(403, self._check_access(None, reverse(
            'list_template_delete', args=('fun.example.com', 'template_id'))))

    def _check_access(self, user, url):
        if user is not None:
            self.client.login(
                username=user.username, password=user.password)
        response = self.client.get(url)
        return response.status_code

    def test_template_index_view_non_owner(self):
        url = reverse('list_template_list', args=('fun.example.com',))
        response = self.client.login(username='testuser', password='testpass')
        self.assertTrue(response)
        response = self.client.get(url)
        self.assertEqual(response.status_code, 403)

    def test_template_index_view_owner(self):
        url = reverse('list_template_list', args=('fun.example.com',))
        response = self.client.login(
            username='list_owner', password='testpass')
        self.assertTrue(response)
        response = self.client.get(url)
        self.assertEqual(response.status_code, 200)

    def test_template_index_view_moderator(self):
        url = reverse('list_template_list', args=('fun.example.com',))
        response = self.client.login(
            username='list_mod', password='testpass')
        self.assertTrue(response)
        response = self.client.get(url)
        self.assertEqual(response.status_code, 403)

    def test_template_index_view_superuser(self):
        url = reverse('list_template_list', args=('fun.example.com',))
        response = self.client.login(
            username='su', password='testpass')
        self.assertTrue(response)
        response = self.client.get(url)
        self.assertEqual(response.status_code, 200)

    def test_template_create_view(self):
        self.client.login(
            username='list_owner', password='testpass')
        # First, let's make sure we can GET the view.
        url = reverse('list_template_new', args=('fun.example.com',))
        response = self.client.get(url)
        self.assertEqual(response.status_code, 200)
        self.assertTrue('Choose the template you want to customize.'
                        in str(response.content))
        # Now, let's try to create a new Template.
        data = {'name': 'list:admin:action:post',
                'data': 'This is test data.'}
        response = self.client.post(url, data=data)
        self.assertEqual(response.status_code, 302)
        self.assertEqual(
            response.url,
            reverse('list_template_list', args=('fun.example.com',)))
        self.assertTrue(EmailTemplate.objects.filter(
            context='list',
            name='list:admin:action:post',
            identifier='fun.example.com').exists())

    def test_template_create_view_whitespace(self):
        self.client.login(
            username='list_owner', password='testpass')
        # First, let's make sure we can GET the view.
        url = reverse('list_template_new', args=('fun.example.com',))
        response = self.client.get(url)
        self.assertEqual(response.status_code, 200)
        self.assertTrue('Choose the template you want to customize.'
                        in str(response.content))
        # Now, let's try to create a new Template with leading whitespace.
        data = {'name': 'list:admin:action:post',
                'data': '\n\n\t\tThis is test data.'}
        response = self.client.post(url, data=data)
        self.assertEqual(response.status_code, 302)
        # Now, let's check to make sure that the data wasn't stripped.
        templates = EmailTemplate.objects.get(
            name='list:admin:action:post', identifier='fun.example.com')
        self.assertEqual(templates.data, '\n\n\t\tThis is test data.')

    def test_template_create_all(self):
        # Test that we can create all the template options.
        self.client.login(
            username='list_owner', password='testpass')
        url = reverse('list_template_new', args=('fun.example.com',))
        for template_name, template_desc in TEMPLATES_LIST:
            data = {'name': template_name,
                    'data': 'This is test data.',
                    'language': 'en'}
            response = self.client.post(url, data=data)
            self.assertEqual(response.status_code, 302, template_name)

    def test_template_unique_property(self):
        # Test that we can create templates with uniqueness.
        self.client.login(username='list_owner', password='testpass')
        url = reverse('list_template_new', args=('fun.example.com',))
        data = {'name': 'list:admin:action:post',
                'data': 'This is test data.'}
        response = self.client.post(url, data=data)
        self.assertEqual(response.status_code, 302)
        template = EmailTemplate.objects.get(
            identifier='fun.example.com', context='list', name=data['name'])
        self.assertEqual(template.data, 'This is test data.')
        # Now, let's create a 2nd domain, and set a template with different
        # domain and same name.
        mlist2 = self.domain.create_list('new')
        url = reverse('list_template_new', args=('new.example.com',))
        # Let's become superuser to post to this.
        self.assertTrue(self.client.login(username='su', password='testpass'))
        response = self.client.post(url, data=data)
        self.assertEqual(response.status_code, 302)
        templates = EmailTemplate.objects.filter(name=data['name'])
        self.assertEqual(len(templates), 2)
        # Clean up.
        try:
            for template in mlist2.templates:
                template.delete()
        except urllib.error.HTTPError:
            pass

    def test_template_delete_view(self):
        self.client.login(
            username='list_owner', password='testpass')
        template = EmailTemplate.objects.create(
            name='list:admin:action:post',
            data='This is some new data.',
            context='list',
            identifier='fun.example.com')
        # First, make sure we get the confirmation page when we GET.
        url = reverse('list_template_delete',
                      args=('fun.example.com', template.id))
        response = self.client.get(url)
        self.assertEqual(response.status_code, 200)
        # Now, let's try to Delete this domain.
        response = self.client.post(url)
        self.assertEqual(response.status_code, 302)
        self.assertEqual(
            response.url,
            reverse('list_template_list', args=('fun.example.com',)))
        # Make sure that this domain is now deleted.
        self.assertFalse(EmailTemplate.objects.filter(
            context='list',
            name='list:admin:action:post',
            identifier='fun.example.com').exists())

    def _test_template_update_view(self, template_name, updated_data):
        self.client.login(
            username='list_owner', password='testpass')
        data = dict(name=template_name,
                    context='list',
                    identifier='fun.example.com',
                    data='This is some new data.')
        template = EmailTemplate.objects.create(**data)
        # First, make sure we can GET the page.
        url = reverse('list_template_update',
                      args=('fun.example.com', template.id))
        response = self.client.get(url)
        self.assertEqual(response.status_code, 200)
        # Now, let's try to update this template by POST'ing.
        data = {'data': updated_data}
        response = self.client.post(url, data)
        self.assertEqual(response.status_code, 302)
        template = EmailTemplate.objects.get(pk=template.id)
        self.assertEqual(template.data, data['data'])

    def test_template_update_view(self):
        self._test_template_update_view('list:admin:action:post',
                                        'This is the update template data.')
        self._test_template_update_view('list:admin:action:subscribe',
                                        '\n\n\n\tHello World')


class TestTemplateAPIView(ViewTestCase):

    def setUp(self):
        super().setUp()
        self.domain = self.mm_client.create_domain('example.com')
        MailDomain.objects.create(
            site=Site.objects.get_current(), mail_domain='example.com')
        self.mlist = self.domain.create_list('fun')

    def tearDown(self):
        EmailTemplate.objects.all().delete()
        super().tearDown()

    def test_get_empty_templates_via_API(self):
        # Test that we can get a domain level template from API.
        data = dict(name='list:admin:action:post',
                    context='domain',
                    identifier='example.com',
                    data='')
        # First, let's create a template.
        EmailTemplate.objects.create(**data)
        # Now, let's try to GET this template.
        url = reverse(
            'rest_template',
            kwargs=dict(
                name=data['name'],
                context=data['context'],
                identifier=data['identifier']
                )
            )
        response = self.client.get(url)
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.content, b'')
        self.assertRegex(response["Content-Type"],
                         'charset=' + settings.DEFAULT_CHARSET)
        self.assertNotRegex(response["Content-Type"], 'charset=$')

    def test_get_one_domain_template_via_API(self):
        # Test that we can get a domain level template from API.
        data = dict(name='list:admin:action:post',
                    context='domain',
                    identifier='example.com',
                    data='This is some new data.')
        # First, let's create a template.
        EmailTemplate.objects.create(**data)
        # Now, let's try to GET this template.
        url = reverse(
            'rest_template',
            kwargs=dict(
                name=data['name'],
                context=data['context'],
                identifier=data['identifier']
            )
        )
        response = self.client.get(url)
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.content, b'This is some new data.')
        self.assertRegex(response["Content-Type"],
                         'charset=' + settings.DEFAULT_CHARSET)
        self.assertNotRegex(response["Content-Type"], 'charset=$')

    def test_get_one_list_template_via_API(self):
        # Test that we can get a list level template from API.
        data = dict(name='list:admin:action:post',
                    context='list',
                    identifier='fun.example.com',
                    data='This is some other new data.')
        # First, let's create a template.
        EmailTemplate.objects.create(**data)
        # Now, let's try to GET this template.
        url = reverse(
            'rest_template',
            kwargs=dict(
                name=data['name'],
                context=data['context'],
                identifier=data['identifier']
            )
        )
        response = self.client.get(url)
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.content, b'This is some other new data.')
        self.assertRegex(response["Content-Type"],
                         'charset=' + settings.DEFAULT_CHARSET)
        self.assertNotRegex(response["Content-Type"], 'charset=$')



# -*- coding: utf-8 -*-
# Copyright (C) 2012-2021 by the Free Software Foundation, Inc.
#
# This file is part of Postorius.
#
# Postorius is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option)
# any later version.
# Postorius is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along with
# Postorius.  If not, see <http://www.gnu.org/licenses/>.


from functools import partial

from django.contrib.auth.models import User
from django.contrib.sites.models import Site
from django.test import override_settings
from django.urls import reverse

from allauth.account.models import EmailAddress
from django_mailman3.models import MailDomain

from postorius.tests.utils import ViewTestCase


class ListIndexPageTest(ViewTestCase):
    """Tests for the list index page."""

    def setUp(self):
        super(ListIndexPageTest, self).setUp()
        self.domain = self.mm_client.create_domain('example.com')
        self.foo_list = self.domain.create_list('foo')
        self.bar_list = self.domain.create_list('bar')

        self.user = User.objects.create_user('user', 'user@example.com', 'pwd')
        self.superuser = User.objects.create_superuser(
            'su', 'su@example.com', 'pwd')

        for user in (self.user, self.superuser):
            EmailAddress.objects.create(
                user=user, email=user.email, verified=True)

    def test_list_index_contains_the_lists(self):
        # The list index page should contain the lists
        response = self.client.get(reverse('list_index'))
        self.assertEqual(response.status_code, 200)
        self.assertEqual(len(response.context['lists']), 2)
        # The lists should be sorted by address
        self.assertEqual(
            [ml.fqdn_listname for ml in response.context['lists']],
            ['bar@example.com', 'foo@example.com'])

    def test_list_index_only_contains_advertised_lists(self):
        # The list index page should contain only contain the advertised lists
        baz_list = self.domain.create_list('baz')
        baz_list.settings['advertised'] = False
        baz_list.settings.save()
        response = self.client.get(reverse('list_index'))
        self.assertEqual(response.status_code, 200)
        self.assertEqual(len(response.context['lists']), 2)
        self.assertNotIn(
            'baz.example.com',
            [ml.list_id for ml in response.context['lists']])

    def test_list_index_post_redirects(self):
        response = self.client.post(reverse('list_index'),
                                    dict(list='foo.example.com'))
        # This should redirect to the list's summary view.
        self.assertEqual(response.status_code, 302)
        self.assertEqual(response.url, '/postorius/lists/foo.example.com/')

    def test_list_index_unadvertized(self):
        # Test that superuser can see unadvertized lists.
        baz_list = self.domain.create_list('baz')
        baz_list.settings['advertised'] = False
        baz_list.settings.save()
        # Superuser should be able to see all lists.
        self.client.login(username='su', password='pwd')
        url = reverse('list_index') + '?all-lists'
        response = self.client.get(url)
        self.assertEqual(len(response.context['lists']), 3)
        self.assertTrue(
            b'Only admins see unadvertised lists in the list index.' in
            response.content)

    def test_list_index_all_lists(self):
        # Test that list index page for a logged-in user.
        self.client.login(username='user', password='pwd')
        url = reverse('list_index')
        response = self.client.get(url)
        # Since this user isn't related to any list, their index page will
        # redirect to all-lists.
        self.assertEqual(response.status_code, 302)
        self.assertEqual(response.url, '/postorius/lists/?all-lists')
        response = self.client.get(response.url)
        self.assertEqual(response.status_code, 200)
        self.assertEqual(len(response.context['lists']), 2)
        # Now, if we subscribe the user to a list, it will show only that list.
        self.foo_list.subscribe(
            self.user.email, pre_confirmed=True, pre_verified=True)
        response = self.client.get(url)
        self.assertEqual(response.status_code, 200)
        self.assertEqual(len(response.context['lists']), 1)
        # List that you are owners and moderators for should also show up.
        self.bar_list.add_owner(self.user.email)
        response = self.client.get(url)
        self.assertEqual(response.status_code, 200)
        self.assertEqual(len(response.context['lists']), 2)

    def test_list_index_owner_only(self):
        # Test that filtering by list-owner role works.
        self.client.login(username='user', password='pwd')
        url = reverse('list_index')
        response = self.client.get(url + '?role=owner')
        # Current user is not an owner of any list.
        self.assertEqual(response.status_code, 200)
        self.assertEqual(len(response.context['lists']), 0)
        # Let's make them an owner of a list and see if it shows up.
        self.foo_list.add_owner(self.user.email)
        response = self.client.get(url + '?role=owner')
        self.assertEqual(response.status_code, 200)
        self.assertEqual(len(response.context['lists']), 1)
        # Test all lists are available on all-list page.
        response = self.client.get(url + '?all-lists')
        self.assertEqual(response.status_code, 200)
        self.assertEqual(len(response.context['lists']), 2)

    def test_list_index_moderator_only(self):
        # Test that filtering by list-moderator role works.
        self.client.login(username='user', password='pwd')
        url = reverse('list_index')
        response = self.client.get(url + '?role=moderator')
        # Current user is not an owner of any list.
        self.assertEqual(response.status_code, 200)
        self.assertEqual(len(response.context['lists']), 0)
        # Let's make them an moderator of a list and see if it shows up.
        self.foo_list.add_moderator(self.user.email)
        response = self.client.get(url + '?role=moderator')
        self.assertEqual(response.status_code, 200)
        self.assertEqual(len(response.context['lists']), 1)
        # Test all lists are available on all-list page.
        response = self.client.get(url + '?all-lists')
        self.assertEqual(response.status_code, 200)
        self.assertEqual(len(response.context['lists']), 2)

    def test_list_index_subscriber_only(self):
        # Test that filtering by list-moderator role works.
        self.client.login(username='user', password='pwd')
        url = reverse('list_index')
        response = self.client.get(url + '?role=member')
        # Current user is not an owner of any list.
        self.assertEqual(response.status_code, 200)
        self.assertEqual(len(response.context['lists']), 0)
        # Let's make them an moderator of a list and see if it shows up.
        self.foo_list.subscribe(
            self.user.email, pre_confirmed=True, pre_verified=True)
        response = self.client.get(url + '?role=member')
        self.assertEqual(response.status_code, 200)
        self.assertEqual(len(response.context['lists']), 1)
        # Test all lists are available on all-list page.
        response = self.client.get(url + '?all-lists')
        self.assertEqual(response.status_code, 200)
        self.assertEqual(len(response.context['lists']), 2)

    def test_list_index_multiple_roles(self):
        # Test that filtering by list-moderator role works.
        self.client.login(username='user', password='pwd')
        url = reverse('list_index')
        # Current user is not an owner of any list.
        self.foo_list.subscribe(
            self.user.email, pre_confirmed=True, pre_verified=True)
        self.foo_list.add_owner(self.user.email)
        response = self.client.get(url)
        self.assertEqual(response.status_code, 200)
        # Make sure that lists aren't repeated if the user has multiple roles
        # in the same mailing list.
        self.assertEqual(len(response.context['lists']), 1)

    def test_list_index_multiple_addresses(self):
        EmailAddress.objects.create(
            user=self.user, email='test-email2@example.com', verified=True)
        self.foo_list.subscribe(
            'test-email2@example.com', pre_confirmed=True, pre_verified=True)
        self.client.login(username='user', password='pwd')
        url = reverse('list_index')
        response = self.client.get(url)
        self.assertEqual(response.status_code, 200)
        # Make sure that memberships of other email addresses than primary one
        # showup here.
        self.assertEqual(len(response.context['lists']), 1)

    def test_list_index_pagination(self):
        # Test that pagination is shown when user is not authenticated.
        url = reverse('list_index')
        response = self.client.get(url)
        self.assertEqual(response.status_code, 200)
        self.assertTrue(b'Results per page' in response.content)
        # Test that pagination is not shown when user is authenticated.
        self.client.login(username='user', password='pwd')
        self.foo_list.add_owner(self.user.email)
        response = self.client.get(url)
        self.assertEqual(response.status_code, 200)
        self.assertEqual(len(response.context['lists']), 1)
        self.assertTrue(b'Results per page' not in response.content)
        # Test that pagination is shown when `all-lists` is in query params.
        response = self.client.get(url + '?all-lists')
        self.assertEqual(response.status_code, 200)
        self.assertTrue(b'Results per page' in response.content)


@override_settings(FILTER_VHOST=True, ALLOWED_HOSTS=["*"])
class DomainFilteringListIndexPageTest(ListIndexPageTest):
    """Tests for the list index page when domain filtering is enabled.

    This also runs all the ListIndexPageTests with FILTER_VHOST enabled for
    more coverage."""

    def setUp(self):
        super(DomainFilteringListIndexPageTest, self).setUp()
        self.domain2 = self.mm_client.create_domain('example.org')
        self.quux_list = self.domain2.create_list('quux')
        self.thud_list = self.domain2.create_list('thud')
        self._site = Site.objects.create(domain='www.example.org',
                                         name='www')
        self.mail_domain2 = MailDomain.objects.create(
            site=self._site, mail_domain="example.org")

        self.client._get = self.client.get
        self.client.get = partial(self.client._get, HTTP_HOST='example.com')
        self.client.get2 = partial(self.client._get, HTTP_HOST='example.org')
        self.client.get3 = partial(self.client._get,
                                   HTTP_HOST='www.example.org')

    def test_domain2_list_index_contains_the_lists(self):
        # The list index page should contain only the requested domain's lists
        response = self.client.get2(reverse('list_index'))
        self.assertEqual(response.status_code, 200)
        self.assertEqual(len(response.context['lists']), 2)
        self.assertEqual(
            [ml.fqdn_listname for ml in response.context['lists']],
            ['quux@example.org', 'thud@example.org'])

    def test_domain2_list_index_www_host_contains_the_lists(self):
        # The list index page should contain only the requested domain's lists
        response = self.client.get3(reverse('list_index'))
        self.assertEqual(response.status_code, 200)
        self.assertEqual(len(response.context['lists']), 2)
        self.assertEqual(
            [ml.fqdn_listname for ml in response.context['lists']],
            ['quux@example.org', 'thud@example.org'])

    def test_domain2_list_index_all_lists(self):
        # Test that list index page for a logged-in user.
        self.client.login(username='user', password='pwd')
        url = reverse('list_index')
        response = self.client.get2(url)
        # Since this user isn't related to any list, their index page will
        # redirect to all-lists.
        self.assertEqual(response.status_code, 302)
        self.assertEqual(response.url, '/postorius/lists/?all-lists')
        response = self.client.get2(response.url)
        self.assertEqual(response.status_code, 200)
        self.assertEqual(len(response.context['lists']), 2)
        # Now, if we subscribe the user to a list, it will show only that list.
        self.quux_list.subscribe(
            self.user.email, pre_confirmed=True, pre_verified=True)
        response = self.client.get2(url)
        self.assertEqual(response.status_code, 200)
        self.assertEqual(len(response.context['lists']), 1)
        # List that you are owners and moderators for should also show up.
        self.thud_list.add_owner(self.user.email)
        response = self.client.get2(url)
        self.assertEqual(response.status_code, 200)
        self.assertEqual(len(response.context['lists']), 2)



# -*- coding: utf-8 -*-
# Copyright (C) 2017-2021 by the Free Software Foundation, Inc.
#
# This file is part of Postorius.
#
# Postorius is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option)
# any later version.
#
# Postorius is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along with
# Postorius.  If not, see <http://www.gnu.org/licenses/>.
#

# from django.test import RequestFactory, TestCase
# from django.urls import reverse

# import mock
# from mailmanclient import MailmanConnectionError

# from postorius.models import MailmanApiError


# class TestMiddleware(TestCase):

#     def setUp(self):
#         self.factory = RequestFactory()

#     @mock.patch('requests.request')
#     @mock.patch('postorius.views.list.list_index')
#     def test_middleware_request(self, mock_request, mock_list_index):
#         # Mock the view function to raise MailmanApiError and verify
#         # the behavior of the middleware function.
#         mock_list_index.side_effect = MailmanApiError
#         response = self.client.get(reverse('list_index'))
#         # Check that correct error page is rendered.
#         self.assertEqual('Mailman REST API not available. Please start Mailman core.',    # noqa
#                          response.context['error'])
#         mock_list_index.reset_mock()
#         # Check similar semantics with MailmanConnectionError from
#         # mailmanclient.
#         mock_list_index.side_effect = MailmanConnectionError
#         response = self.client.get(reverse('list_index'))
#         self.assertEqual('Mailman REST API not available. Please start Mailman core.',    # noqa
#                          response.context['error'])



# -*- coding: utf-8 -*-
# Copyright (C) 2017-2021 by the Free Software Foundation, Inc.
#
# This file is part of Postorius.
#
# Postorius is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option)
# any later version.
#
# Postorius is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along with
# Postorius.  If not, see <http://www.gnu.org/licenses/>.
#

from django.contrib.sites.models import Site
from django.test import TestCase

from postorius.forms.domain_forms import (
    DomainEditForm, DomainForm, DomainOwnerForm)


class TestDomainEditForm(TestCase):
    def test_form_does_not_contain_mail_host(self):
        form = DomainEditForm()
        self.assertTrue('description' in form.fields)
        self.assertTrue('alias_domain' in form.fields)
        self.assertTrue('site' in form.fields)
        self.assertFalse('mail_host' in form.fields)


class TestDomainOwnerAddForm(TestCase):
    def test_form_sanity(self):
        form = DomainOwnerForm(dict(email='some'))
        self.assertFalse(form.is_valid())
        self.assertIn('email', form.errors)
        form = DomainOwnerForm(dict(email='some@example.com'))
        self.assertTrue(form.is_valid())


class TestDomainForm(TestCase):
    def test_form_contains_mail_host(self):
        form = DomainForm()
        self.assertTrue('description' in form.fields)
        self.assertTrue('alias_domain' in form.fields)
        self.assertTrue('site' in form.fields)
        self.assertTrue('mail_host' in form.fields)

    def test_form_labels(self):
        form = DomainForm()
        self.assertTrue(form.fields['mail_host'].label == 'Mail Host')
        self.assertTrue(form.fields['description'].label == 'Description')
        self.assertTrue(form.fields['alias_domain'].label == 'Alias Domain')
        self.assertTrue(form.fields['site'].label == 'Web Host')

    def test_error_messages(self):
        form = DomainForm({
            'mail_host': 'mailman.most-desirable.org',
        })
        self.assertFalse(form.is_valid())
        self.assertTrue('site' in form.errors.keys())
        self.assertEqual(form.errors['site'][0], 'Please enter a domain name')
        form = DomainForm({'site': 1})
        self.assertFalse(form.is_valid())
        self.assertTrue('mail_host' in form.errors.keys())
        self.assertEqual(form.errors['mail_host'][0],
                         'Please enter a domain name')

    def test_form_fields_validation(self):
        # With all valid values, the form should be valid.
        form = DomainForm({
            'mail_host': 'mailman.most-desirable.org',
            'description': 'The Most Desirable organization',
            'site': 1,
        })
        self.assertTrue(form.is_valid())
        # With a valid alias_domain the form should be valid.
        form = DomainForm({
            'mail_host': 'mailman.most-desirable.org',
            'description': 'The Most Desirable organization',
            'alias_domain': 'x.most-desirable.org',
            'site': 1,
        })
        self.assertTrue(form.is_valid())
        # Because there is no site_id 2 by default in Django, this form should
        # not be valid.
        form = DomainForm({
            'mail_host': 'mailman.most-desirable.org',
            'description': 'The Most Desirable organization',
            'site': 2,
        })
        self.assertFalse(form.is_valid())
        self.assertTrue('site' in form.errors.keys())
        self.assertEqual(form.errors['site'][0],
                         'Select a valid choice.'
                         ' That choice is not one of the available choices.')
        # Now we use an invalid value for domain name.
        form = DomainForm({
            'mail_host': 'mailman@most-desirable.org',
            'description': 'The Most Desirable organization',
            'site': 1,
        })
        self.assertFalse(form.is_valid())
        self.assertTrue('mail_host' in form.errors.keys())
        self.assertEqual(form.errors['mail_host'][0],
                         'Please enter a valid domain name')
        # Now we use an invalid value for alias domain.
        form = DomainForm({
            'mail_host': 'mailman.most-desirable.org',
            'description': 'The Most Desirable organization',
            'alias_domain': 'x@most-desirable.org',
            'site': 1,
        })
        self.assertFalse(form.is_valid())
        self.assertTrue('alias_domain' in form.errors.keys())
        self.assertEqual(form.errors['alias_domain'][0],
                         'Please enter a valid domain name or nothing.')

    def test_site_field_values(self):
        form = DomainForm()
        self.assertTrue('site' in form.fields.keys())
        self.assertTrue([x for x in form.fields['site'].choices],
                        [(1, 'example.com (example.com)')])
        # Now let's create a new domain and see if it shows up.
        Site.objects.create(domain='mail.most-desirable.org', name='My Domain')
        Site.objects.create(domain='dom.most-desirable.org', name='A Domain')
        # Items are ordered by "name".
        self.assertTrue([x for x in form.fields['site'].choices],
                        [(1, 'A Domain (dom.most-desirable.org)'),
                         (2, 'example.com (example.com)'),
                         (3, 'My Domain (mail.most-desirable.org)')])
        # Initial value should be set to the current site.
        self.assertEqual(form.fields['site'].initial(),
                         Site.objects.get(domain='example.com'))



# -*- coding: utf-8 -*-
# Copyright (C) 2018-2021 by the Free Software Foundation, Inc.
#
# This file is part of Postorius.
#
# Postorius is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option)
# any later version.
#
# Postorius is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along with
# Postorius.  If not, see <http://www.gnu.org/licenses/>.
#
# flake8: noqa

from django.utils.translation import gettext_lazy as _


TEMPLATES_LIST = (
    ('list:admin:action:post',
     _('Sent to the list administrators when moderator approval for a posting is required.')),
    ('list:admin:action:subscribe',
     _('Sent to the list administrators when moderator approval for a subscription request is required.')),
    ('list:admin:action:unsubscribe',
     _('Sent to the list administrators when moderator approval for an unsubscription request is required.')),
    ('list:admin:notice:subscribe',
     _('Sent to the list administrators to notify them when a new member has been subscribed.')),
    ('list:admin:notice:unrecognized',
     _('Sent to the list administrators when a bounce message in an unrecognized format has been received.')),
    ('list:admin:notice:disable',
     _('Sent to the list administrators to notify them when a member\'s delivery is disabled due to excessive bounces.')),
    ('list:admin:notice:unsubscribe',
     _('Sent to the list administrators to notify them when a member has been unsubscribed.')),
    ('list:member:digest:footer',
     _('The footer for a digest message.')),
    ('list:member:digest:header',
     _('The header for a digest message.')),
    ('list:member:digest:masthead',
     _('The digest “masthead”; i.e. a common introduction for all digest messages.')),
    ('list:member:regular:footer',
     _('The footer for a regular (non-digest) message.')),
    ('list:member:regular:header',
     _('The header for a regular (non-digest) message.')),
    ('list:user:action:subscribe',
     _('The message sent to subscribers when a subscription confirmation is required.')),
    ('list:user:action:unsubscribe',
     _('The message sent to subscribers when an unsubscription confirmation is required.')),
    ('list:user:action:invite',
     _('The message sent to subscribers when they are invited to join a list.')),
    ('list:user:notice:goodbye',
     _('The notice sent to a member when they unsubscribe from a mailing list.')),
    ('list:user:notice:hold',
     _('The notice sent to a poster when their message is being held for moderator approval.')),
    ('list:user:notice:no-more-today',
     _('Sent to a user when the maximum number of autoresponses has been reached for that day.')),
    ('list:user:notice:post',
     _('Notice sent to a poster when their message has been received by the mailing list.')),
    ('list:user:notice:probe',
     _('A bounce probe sent to a member when their subscription has been disabled due to bounces.')),
    ('list:user:notice:refuse',
     _('Notice sent to a poster when their message has been rejected by the list’s moderator.')),
    ('list:user:notice:rejected',
     _('Notice sent to a poster when their message has been rejected automatically.')),
    ('list:user:notice:welcome',
     _('The notice sent to a member when they are subscribed to the mailing list.')),
    ('list:user:notice:warning',
     _('The notice sent to a member when their membership has been disabled due to excessive bounces.')),
)



# -*- coding: utf-8 -*-
# Copyright (C) 2017-2021 by the Free Software Foundation, Inc.
#
# This file is part of Postorius.
#
# Postorius is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option)
# any later version.
#
# Postorius is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along with
# Postorius.  If not, see <http://www.gnu.org/licenses/>.
#
from django.test import RequestFactory, TestCase

from postorius.utils import render_api_error


class TestUtils(TestCase):

    def setUp(self):
        self.factory = RequestFactory()

    def test_render_api_error_works(self):
        request = self.factory.get('/postorius/lists')
        response = render_api_error(request)
        self.assertTrue('Mailman REST API not available.' in
                        str(response.content))
        self.assertEqual(response.status_code, 503)



# -*- coding: utf-8 -*-
# Copyright (C) 2017-2021 by the Free Software Foundation, Inc.
#
# This file is part of Postorius.
#
# Postorius is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option)
# any later version.
#
# Postorius is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along with
# Postorius.  If not, see <http://www.gnu.org/licenses/>.
#

from django.contrib.auth.models import User

from allauth.account.models import EmailAddress
from mailmanclient import MailingList

from postorius.models import Mailman404Error
from postorius.templatetags.membership_helpers import (
    get_list, user_is_list_moderator, user_is_list_owner)
from postorius.tests.utils import ViewTestCase


class TestMembershipHelpers(ViewTestCase):

    def setUp(self):
        super(TestMembershipHelpers, self).setUp()
        # Create a domain.
        self.domain = self.mm_client.create_domain('example.com')
        self.mlist = self.domain.create_list('test_list')
        self.test_user = User.objects.create_user(
            'test_user', 'test_user@example.com', 'pwd')
        self.test_superuser = User.objects.create_superuser(
            'test_superuser', 'test_superuser@example.com', 'pwd')
        self.test_owner = User.objects.create_user(
            'testowner', 'owner@example.com', 'testpass')
        self.test_moderator = User.objects.create_user(
            'testmoderator', 'moderator@example.com', 'testpass')
        for user in (self.test_user, self.test_superuser, self.test_owner,
                     self.test_moderator):
            EmailAddress.objects.create(
                user=user, email=user.email, verified=True)
        self.mlist.add_owner(self.test_owner.email)
        self.mlist.add_moderator(self.test_moderator.email)

    def tearDown(self):
        self.domain.delete()
        for user in (self.test_user, self.test_superuser, self.test_owner,
                     self.test_moderator):
            user.delete()

    def test_get_list(self):
        # Given the list's posting address, get_list should return the
        # MailingList object.
        mlist = get_list('test_list@example.com')
        self.assertTrue(isinstance(mlist, MailingList))
        self.assertEqual(mlist.fqdn_listname, 'test_list@example.com')
        # Given a list's fqdn, it should be possible to get the mlist too.
        mlist = get_list('test_list.example.com')
        self.assertTrue(isinstance(mlist, MailingList))
        self.assertEqual(mlist.fqdn_listname, 'test_list@example.com')
        # Given a MailingList object, make sure it is returned back.
        res = get_list(mlist)
        self.assertTrue(res is mlist)
        # Given wrong MailingList identifier should raise proper error.
        # TODO (maxking): Ideally, this should raise a validation error, given
        # that the MailingList identifier is wrong. However, given the error
        # propagation right now, it doesn't seem to be possible.
        with self.assertRaises(Mailman404Error):
            mlist = get_list('example.com')

    def test_user_is_list_owner(self):
        # First, it should return false as the user is not a list owner.
        self.assertFalse(user_is_list_owner(self.test_user, self.mlist))
        # It should return True for the list owner.
        self.assertTrue(user_is_list_owner(self.test_owner, self.mlist))
        # It should return False for the list moderator.
        self.assertFalse(user_is_list_owner(self.test_moderator, self.mlist))
        # Now let's add the test_user as an owner of the list.
        self.mlist.add_owner(self.test_user.email)
        self.assertTrue(user_is_list_owner(self.test_user, self.mlist))

    def test_user_is_list_moderator(self):
        # First, it should return false as the user is not a list moderator.
        self.assertFalse(user_is_list_moderator(self.test_user, self.mlist))
        # It should return False for the list owner.
        self.assertFalse(user_is_list_moderator(self.test_owner, self.mlist))
        # It should return False for the list moderator.
        self.assertTrue(user_is_list_moderator(self.test_moderator,
                                               self.mlist))
        # Now let's add the test_user as an owner of the list.
        self.mlist.add_moderator(self.test_user.email)
        self.assertTrue(user_is_list_moderator(self.test_user, self.mlist))



# -*- coding: utf-8 -*-
# Copyright (C) 2018-2021 by the Free Software Foundation, Inc.
#
# This file is part of Postorius.
#
# Postorius is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option)
# any later version.
#
# Postorius is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along with
# Postorius.  If not, see <http://www.gnu.org/licenses/>.

from io import StringIO

from django.core.management import call_command
from django.test import TestCase


class TestMigrationPending(TestCase):

    def test_pending_migration(self):
        # Test that there aren't any pending migrations to be added.
        output = StringIO()
        # We will call the makemigrations command and check the output.
        call_command(
            'makemigrations', interactive=False, dry_run=True, stdout=output)
        self.assertEqual(output.getvalue(), 'No changes detected\n')



# -*- coding: utf-8 -*-
# Copyright (C) 2018-2021 by the Free Software Foundation, Inc.
#
# This file is part of Postorius.
#
# Postorius is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option)
# any later version.
#
# Postorius is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along with
# Postorius.  If not, see <http://www.gnu.org/licenses/>.
#


from django.contrib.auth.models import AnonymousUser, User
from django.contrib.sites.models import Site
from django.core.exceptions import PermissionDenied
from django.http import HttpResponse
from django.test import RequestFactory
from django.urls import reverse
from django.views.generic import TemplateView

from allauth.account.models import EmailAddress
from django_mailman3.models import MailDomain

from postorius.auth.mixins import (
    DomainOwnerMixin, ListModeratorMixin, ListOwnerMixin)
from postorius.tests.utils import ViewTestCase


class TestAuthenticationMixins(ViewTestCase):

    def setUp(self):
        super().setUp()
        self.domain = self.mm_client.create_domain('example.com')
        self.su = User.objects.create_superuser(
            username='su', email='su@example.com', password='testpass')
        self.do = User.objects.create_user(
            username='domain_owner', email='do@example.com',
            password='testpass')
        self.list_owner = User.objects.create_user(
            username='list_owner', email='owner@example.com',
            password='testpass')
        self.list_mod = User.objects.create_user(
            username='list_mod', email='mod@example.com',
            password='testpass')
        self.testuser = User.objects.create_user(
            username='testuser', email='testuser@example.com',
            password='testpass')
        for user in (
                self.testuser,
                self.su,
                self.list_owner,
                self.list_mod,
                self.do
        ):
            EmailAddress.objects.create(
                user=user, email=user.email, verified=True)
        MailDomain.objects.create(
            site=Site.objects.get_current(), mail_domain='example.com')
        self.domain.add_owner('do@example.com')
        self.mlist = self.domain.create_list('fun')
        self.mlist.add_owner('owner@example.com')
        self.mlist.add_moderator('mod@example.com')
        self.factory = RequestFactory()

    def test_list_owner_mixin(self):
        class TestView(ListOwnerMixin, TemplateView):
            def get(request, *args, **kwargs):
                return HttpResponse('Hello')
        view = TestView.as_view()
        request = self.factory.get(
            reverse('list_delete', args=('fun.example.com',)))
        # Make sure that an anonymous user isn't able to access this view.
        request.user = AnonymousUser()
        with self.assertRaises(PermissionDenied):
            view(request)
        # Make sure a logged-in user who is not an owner isn't able to access
        # this view.
        request.user = self.list_mod
        with self.assertRaises(PermissionDenied):
            view(request, list_id='fun.example.com')
        # Make sure that a superuser is able to access the view too.
        request.user = self.su
        response = view(request, list_id='fun.example.com')
        self.assertEqual(response.content, b'Hello')
        # Make sure that a list owner is able to access this view.
        request.user = self.list_owner
        response = view(request, list_id='fun.example.com')
        self.assertEqual(response.content, b'Hello')

    def test_list_moderator_mixin(self):
        class TestView(ListModeratorMixin, TemplateView):
            def get(request, *args, **kwargs):
                return HttpResponse('Hello')
        view = TestView.as_view()
        request = self.factory.get(
            reverse('list_delete', args=('fun.example.com',)))
        # Make sure that an anonymous user can not get this.
        request.user = AnonymousUser()
        with self.assertRaises(PermissionDenied):
            view(request)
        # Make sure a logged-in user who is not an owner isn't able to access
        # this view.
        request.user = self.testuser
        with self.assertRaises(PermissionDenied):
            view(request, list_id='fun.example.com')
        # Make sure that a superuser is able to access the view too.
        request.user = self.su
        response = view(request, list_id='fun.example.com')
        self.assertEqual(response.content, b'Hello')
        # Make sure that a list owner is able to access this view.
        request.user = self.list_owner
        response = view(request, list_id='fun.example.com')
        self.assertEqual(response.content, b'Hello')
        # Make sure that a list moderator is able to access this view.
        request.user = self.list_mod
        response = view(request, list_id='fun.example.com')
        self.assertEqual(response.content, b'Hello')

    def test_domain_owner_requred(self):
        class TestView(DomainOwnerMixin, TemplateView):
            def get(request, *args, **kwargs):
                return HttpResponse('Hello')
        view = TestView.as_view()
        request = self.factory.get(
            reverse('domain_delete', args=('example.com',)))
        # Non logged-in user.
        request.user = AnonymousUser()
        with self.assertRaises(PermissionDenied):
            view(request, domain='example.com')
        # Logged-in normal user.
        request.user = self.testuser
        with self.assertRaises(PermissionDenied):
            view(request, domain='example.com')
        # Another mailing list owner.
        request.user = self.list_owner
        with self.assertRaises(PermissionDenied):
            view(request, domain='example.com')
        # Test that superuser is able to access the page.
        request.user = self.su
        response = view(request, domain='example.com')
        self.assertEqual(response.content, b'Hello')
        # The domain owner itself should be able to access everything.
        request.user = self.do
        response = view(request, domain='example.com')
        self.assertEqual(response.content, b'Hello')



# -*- coding: utf-8 -*-
# Copyright (C) 2019-2021 by the Free Software Foundation, Inc.
#
# This file is part of Postorius.
#
# Postorius is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option)
# any later version.
#
# Postorius is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along with
# Postorius.  If not, see <http://www.gnu.org/licenses/>.
#

import time

from postorius.templatetags.nav_helpers import (
    held_count, pending_subscriptions)
from postorius.tests.utils import ViewTestCase


class TestNavigationHelpers(ViewTestCase):

    def setUp(self):
        super().setUp()
        # Create a domain.
        self.domain = self.mm_client.create_domain('example.com')
        self.mlist = self.domain.create_list('test_list')

    def tearDown(self):
        self.domain.delete()

    def test_subscription_request_count(self):
        # Initially, the count of held messages is 0.
        self.assertEqual(pending_subscriptions(self.mlist), 0)
        self.mlist.settings['subscription_policy'] = 'moderate'
        self.mlist.settings.save()
        self.mlist.subscribe('needsmoderation@example.com',
                             pre_verified=True)
        self.assertEqual(pending_subscriptions(self.mlist), 1)
        # Make sure the ones pending user approval don't show up.
        self.mlist.settings['subscription_policy'] = 'confirm'
        self.mlist.settings.save()
        self.mlist.subscribe('needsconfirmation@example.com',
                             pre_verified=True)
        self.assertEqual(pending_subscriptions(self.mlist), 1)

    def test_held_message_count(self):
        # Initially, the count of held messages is 0.
        self.assertEqual(held_count(self.mlist), 0)
        # Now, let's inject some message.
        msg = """\
From: nonmember@example.com
To: test_list@example.com
Subject: What??
Message-ID: <moderated_01>

Hello.
"""
        inq = self.mm_client.queues['in']
        inq.inject('test_list.example.com', msg)
        # Wait for the message to be processed.
        while True:
            if len(inq.files) == 0:
                break
            time.sleep(0.1)
        # Wait for message to show up in held queue.
        while True:
            all_held = self.mlist.held
            if len(all_held) > 0:
                break
            time.sleep(0.1)

        self.assertEqual(held_count(self.mlist), 1)



# -*- coding: utf-8 -*-
# Copyright (C) 2012-2021 by the Free Software Foundation, Inc.
#
# This file is part of Postorius.
#
# Postorius is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option)
# any later version.
# Postorius is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along with
# Postorius.  If not, see <http://www.gnu.org/licenses/>.


from django.contrib.auth.models import User
from django.test import TestCase
from django.urls import reverse

from allauth.account.models import EmailAddress

from postorius.forms import DomainForm


class DomainViewTest(TestCase):

    def setUp(self):
        self.user = User.objects.create_superuser('su', 'su@example.com',
                                                  'pass')
        EmailAddress.objects.create(
            user=self.user, email=self.user.email, verified=True)

    def tearDown(self):
        self.user.delete()

    def test_form_is_rendered(self):
        self.client.login(username='su', password='pass')
        response = self.client.get(reverse('domain_new'), follow=True)
        self.assertEqual(response.status_code, 200)
        self.assertIsInstance(response.context['form'], DomainForm)



# -*- coding: utf-8 -*-
# Copyright (C) 2012-2021 by the Free Software Foundation, Inc.
#
# This file is part of Postorius.
#
# Postorius is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option)
# any later version.
# Postorius is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along with
# Postorius.  If not, see <http://www.gnu.org/licenses/>.


from django.test import TestCase

from postorius.forms import MemberForm


class TestMemberForm(TestCase):

    def test_form_labels(self):
        form = MemberForm({})
        self.assertTrue('email' in form.fields.keys())
        self.assertEqual(form.fields['email'].label, 'Email Address')
        self.assertTrue('display_name' in form.fields.keys())
        self.assertEqual(form.fields['display_name'].label, 'Display Name')

    def test_form_errors(self):
        form = MemberForm({})
        self.assertFalse(form.is_valid())
        self.assertEqual(form.errors['email'][0],
                         'Please enter an email address.')
        form = MemberForm({'email': 'invalid.example.com'})
        self.assertFalse(form.is_valid())
        self.assertEqual(form.errors['email'][0],
                         'Please enter a valid email address.')



# -*- coding: utf-8 -*-
# Copyright (C) 2019-2021 by the Free Software Foundation, Inc.
#
# This file is part of Postorius.
#
# Postorius is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option)
# any later version.
#
# Postorius is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along with
# Postorius.  If not, see <http://www.gnu.org/licenses/>.

import time

from django.contrib.auth.models import User
from django.urls import reverse

from allauth.account.models import EmailAddress

from postorius.tests.utils import ViewTestCase


class HeldMessageTest(ViewTestCase):
    """Test view for Held message"""

    def setUp(self):
        super().setUp()
        self.domain = self.mm_client.create_domain('example.com')
        self.superuser = User.objects.create_superuser('su', 'su@example.com',
                                                       'pwd')
        EmailAddress.objects.create(user=self.superuser,
                                    email=self.superuser.email,
                                    verified=True)
        self.client.login(username='su', password='pwd')

    def _wait_for_processing(self, queue):
        while True:
            if len(queue.files) == 0:
                break
            time.sleep(0.1)

    def _wait_for_held_message(self, mlist):
        # Wait for the message to land in held queue.
        while True:
            all_held = mlist.held
            if len(all_held) > 0:
                break
            time.sleep(0.1)

    def test_accept_held_messages(self):
        # Test that held messages are visible.
        mlist = self.domain.create_list('test-1')
        test_msg = """\
From: aperson@example.com
To: test-1@example.com
Subject: This is a test message
Message-ID: <test-msg@id>

This is a test message.

"""
        inque = self.mm_client.queues['in']
        inque.inject('test-1.example.com', test_msg)
        # Wait for the message to be formatted.
        self._wait_for_processing(inque)
        # Wait for message to land in held message queue.
        self._wait_for_held_message(mlist)
        # Check that there is a held message.
        self.assertEqual(len(mlist.held), 1)
        held_message = mlist.held[0]
        # Test that a held message is accepted when POST'ing.
        response = self.client.post(
            reverse('moderate_held_message', args=('test-1.example.com', )),
            {'msgid': held_message.request_id,
             'accept': True,
             'moderation_choice': 'no-action'}
            )
        self.assertEqual(response.status_code, 302)

    def test_accept_held_message_moderate_member(self):
        # Test that we can moderate a member and accept a message at the same
        # time.
        mlist = self.domain.create_list('test-2')
        # Set default action to be hold for member's post.
        mlist.settings['default_member_action'] = 'hold'
        mlist.settings.save()
        # Subscribe this user.
        mlist.subscribe('aperson@example.com',
                        pre_verified=True, pre_confirmed=True)
        test_msg = """\
From: aperson@example.com
To: test-2@example.com
Subject: This is a test message
Message-ID: <test-msg@id>

This is a test message.

"""
        inque = self.mm_client.queues['in']
        inque.inject('test-2.example.com', test_msg)
        # Wait for the message to be formatted.
        self._wait_for_processing(inque)
        # Wait for message to land in held message queue.
        self._wait_for_held_message(mlist)
        # Check that there is a held message.
        self.assertEqual(len(mlist.held), 1)
        held_message = mlist.held[0]
        # Test that a held message is accepted when POST'ing.
        response = self.client.post(
            reverse('moderate_held_message', args=('test-2.example.com', )),
            {'msgid': held_message.request_id,
             'accept': True,
             'moderation_choice': 'defer'}
        )
        self.assertEqual(response.status_code, 302)
        member = mlist.get_member('aperson@example.com')
        self.assertEqual(member.moderation_action, 'defer')

    def test_moderate_held_message_missing(self):
        self.domain.create_list('test-3')
        response = self.client.post(
            reverse('moderate_held_message', args=('test-3.example.com', )),
            {'msgid': '12345',
             'accept': True,
             'moderation_choice': 'defer'})

        self.assertHasErrorMessage(response)
        self.assertEqual(response.status_code, 302)
        self.assertEqual(response.url,
                         '/postorius/lists/test-3.example.com/held_messages')

    def test_reject_held_message_with_reason(self):
        mlist = self.domain.create_list('test-2')
        test_msg = """\
From: aperson@example.com
To: test-2@example.com
Subject: This is a test message
Message-ID: <test-msg@id>

This is a test message.

"""
        inque = self.mm_client.queues['in']
        inque.inject('test-2.example.com', test_msg)
        # Wait for the message to be formatted.
        self._wait_for_processing(inque)
        # Wait for message to land in held message queue.
        self._wait_for_held_message(mlist)
        # Check that there is a held message.
        self.assertEqual(len(mlist.held), 1)
        held_message = mlist.held[0]
        # Test that a held message is rejected when POST'ing.
        response = self.client.post(
            reverse('moderate_held_message', args=('test-2.example.com', )),
            {'msgid': held_message.request_id,
             'reject': True,
             'reason': 'Wrong mailing list'}
        )
        self.assertEqual(response.status_code, 302)
        self.assertEqual(len(mlist.held), 0)

    def test_moderate_held_message_set_moderation_for_nonmember(self):
        mlist = self.domain.create_list('test-3')
        test_msg = """\
From: aperson@example.com
To: test-3@example.com
Subject: This is a test message
Message-ID: <test-msg@id>

This is a test message.

"""
        inque = self.mm_client.queues['in']
        inque.inject('test-3.example.com', test_msg)
        # Wait for the message to be formatted.
        self._wait_for_processing(inque)
        # Wait for message to land in held message queue.
        self._wait_for_held_message(mlist)
        # Check that there is a held message.
        self.assertEqual(len(mlist.held), 1)
        held_message = mlist.held[0]
        # Test that a held message is rejected when POST'ing.
        response = self.client.post(
            reverse('moderate_held_message', args=('test-3.example.com', )),
            {'msgid': held_message.request_id,
             'reject': True,
             'reason': 'Wrong mailing list',
             'moderation_choice': 'hold'}
        )
        self.assertEqual(response.status_code, 302)
        self.assertHasSuccessMessage(response, count=2)
        self.assertEqual(len(mlist.held), 0)
        nonmember = mlist.get_nonmember('aperson@example.com')
        self.assertEqual(nonmember.moderation_action, 'hold')

    def test_moderate_held_message_set_moderation_for_member(self):
        mlist = self.domain.create_list('test-4')
        member = mlist.subscribe(
            'aperson@example.com',
            pre_verified=True, pre_approved=True, pre_confirmed=True)
        mlist.settings['default_member_action'] = 'hold'
        mlist.settings.save()
        self.assertIsNotNone(member)
        test_msg = """\
From: aperson@example.com
To: test-4@example.com
Subject: This is a test message
Message-ID: <test-msg1234@id>

This is a test message.
"""
        inque = self.mm_client.queues['in']
        inque.inject('test-4.example.com', test_msg)
        # Wait for the message to be formatted.
        self._wait_for_processing(inque)
        # Wait for message to land in held message queue.
        self._wait_for_held_message(mlist)
        # Check that there is a held message.
        self.assertEqual(len(mlist.held), 1)
        held_message = mlist.held[0]
        # Test that a held message is rejected when POST'ing.
        response = self.client.post(
            reverse('moderate_held_message', args=('test-4.example.com', )),
            {'msgid': held_message.request_id,
             'reject': True,
             'reason': 'Wrong mailing list',
             'moderation_choice': 'defer'}
        )
        self.assertEqual(response.status_code, 302)
        self.assertHasSuccessMessage(response, count=2)
        self.assertEqual(len(mlist.held), 0)
        member = mlist.get_member('aperson@example.com')
        self.assertEqual(member.moderation_action, 'defer')


class TestConfirmToken(ViewTestCase):

    def setUp(self):
        super().setUp()
        self.domain = self.mm_client.create_domain('example.com')
        self.mlist = self.domain.create_list('mylist')
        settings = self.mlist.settings
        settings['subscription_policy'] = 'confirm'
        settings.save()

    def tearDown(self):
        self.mlist.delete()
        self.domain.delete()

    def test_confirm_token(self):
        # Test that we can first get a token page to confirm.
        token = self.mlist.subscribe('aperson@example.com', 'Anne Person')
        self.assertIsNotNone(token.get('token'))
        resp = self.client.get(
            reverse('confirm_token', args=(self.mlist.list_id, )) +
            '?token={}'.format(token.get('token')))
        self.assertEqual(resp.status_code, 200)
        self.assertTrue(b'Confirm subscription of Anne Person'
                        in resp.content)
        self.assertTrue(token.get('token') in resp.content.decode('utf-8'))
        # Now, let's confirm the token.
        with self.assertRaises(ValueError):
            self.mlist.get_member('aperson@example.com')
        resp = self.client.post(
            reverse('confirm_token', args=(self.mlist.list_id,)),
            data={'token': token.get('token')})
        self.assertTrue(resp.status_code, 301)
        member = self.mlist.get_member('aperson@example.com')
        self.assertIsNotNone(member)

    def test_invalid_expired_token(self):
        resp = self.client.get(
            reverse('confirm_token', args=(self.mlist.list_id, )) +
            '?token=badtoken')
        self.assertEqual(resp.status_code, 404)
        self.assertTrue(b'Token expired or invalid' in resp.content)

    def test_moderator_token(self):
        # Set the subscription policy to moderate for confirmation from mod.
        settings = self.mlist.settings
        settings['subscription_policy'] = 'moderate'
        settings.save()
        # Subscribe to generate a mod approval req.
        token = self.mlist.subscribe(
            'aperson@example.com', 'Anne Person', pre_verified=True)
        self.assertIsNotNone(token.get('token'))
        token = self.mlist.get_request(token.get('token'))
        self.assertEqual(token['token_owner'], 'moderator')
        resp = self.client.get(
            reverse('confirm_token', args=(self.mlist.list_id, )) +
            '?token={}'.format(token.get('token')))
        self.assertEqual(resp.status_code, 302)
        self.assertEqual(
            resp.url,
            '/postorius/lists/mylist.example.com/subscription_requests')

    def test_subscribe_unsubscribe_token(self):
        # TODO: This test requires the unsubscription requests to be exposed
        # and hence next version of Mailman Core 3.3.3

        # settings = self.mlist.settings
        # settings['unsubscription_policy'] = 'moderate'
        # settings.save()
        # # Subscribe the address.
        # self.mlist.subscribe(
        #     'aperson@example.com', 'Anne Person',
        #     pre_verified=True, pre_confirmed=True, pre_approved=True)
        # member = self.mlist.get_member('aperson@example.com')
        # self.assertIsNotNone(member)
        # # Unsubscribe the user.
        # self.mlist.unsubscribe('aperson@example.com')
        # member = self.mlist.get_member('aperson@example.com')
        # requests = self.mlist.get_requests(token_owner='moderator')
        # self.assertEqual(len(requests), 1)
        pass



# -*- coding: utf-8 -*-
# Copyright (C) 2012-2021 by the Free Software Foundation, Inc.
#
# This file is part of Postorius.
#
# Postorius is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option)
# any later version.
# Postorius is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along with
# Postorius.  If not, see <http://www.gnu.org/licenses/>.


from django.contrib.auth.models import User
from django.contrib.sites.models import Site
from django.urls import reverse

from allauth.account.models import EmailAddress
from django_mailman3.models import MailDomain

from postorius.tests.utils import ViewTestCase


class DomainCreationTest(ViewTestCase):
    """Tests for the new list page."""

    def setUp(self):
        super(DomainCreationTest, self).setUp()
        self.user = User.objects.create_user('user', 'user@example.com', 'pwd')
        self.superuser = User.objects.create_superuser('su', 'su@example.com',
                                                       'pwd')
        for user in (self.user, self.superuser):
            EmailAddress.objects.create(
                user=user, email=user.email, verified=True)

    def test_permission_denied(self):
        self.client.login(username='user', password='pwd')
        response = self.client.get(reverse('domain_new'))
        self.assertEqual(response.status_code, 403)

    def test_new_domain_created_with_owner(self):
        self.client.login(username='su', password='pwd')
        post_data = {'mail_host': 'example.com',
                     'description': 'A new Domain.',
                     'site': '1',
                     }
        response = self.client.post(reverse('domain_new'), post_data)

        self.assertHasSuccessMessage(response)
        self.assertRedirects(response, reverse('domain_index'))

        a_new_domain = self.mm_client.get_domain('example.com')
        self.assertEqual(a_new_domain.mail_host, 'example.com')
        self.assertEqual(a_new_domain.owners[0].user_id,
                         self.mm_client.get_user('su@example.com').user_id)
        self.assertTrue(
            MailDomain.objects.filter(mail_domain='example.com').exists())
        a_new_domain.delete()

    def test_validation_of_mail_host(self):
        self.client.login(username='su', password='pwd')
        post_data = {'mail_host': 'example com',
                     'description': 'A new Domain',
                     'site': '1',
                     }
        response = self.client.post(reverse('domain_new'), post_data)
        self.assertContains(response, 'Please enter a valid domain name')
        # self.assertHasErrorMessage(response)
        self.assertEqual(response.status_code, 200)

    def test_only_redirect_if_domain_creation_is_successful(self):
        self.client.login(username='su', password='pwd')
        post_data = {'mail_host': 'example.com',
                     'site': '1',
                     'description': 'A new Domain.'}
        response = self.client.post(reverse('domain_new'), post_data)
        self.assertEqual(response.status_code, 302)
        self.assertHasSuccessMessage(response)
        response = self.client.post(reverse('domain_new'), post_data)
        self.assertEqual(response.status_code, 200)
        self.assertIsNotNone(response.context['form'].errors)

    def test_new_domain_updates_site(self):
        # Test that creating a brand new domain updates the associated Site if
        # it is example.com
        self.client.login(username='su', password='pwd')
        site = Site.objects.get(pk=1)
        self.assertEqual(site.domain, 'example.com')
        self.assertEqual(site.name, 'example.com')
        # Any domain other than example.com will update the Site object.
        post_data = {'mail_host': 'otherthan.example.com',
                     'site': '1',
                     'description': 'A new Domain.'}
        response = self.client.post(reverse('domain_new'), post_data)
        self.assertEqual(response.status_code, 302)
        self.assertHasSuccessMessage(response)
        # Get the Site id 1, which was used above.
        site = Site.objects.get(pk=1)
        self.assertEqual(site.domain, 'otherthan.example.com')
        self.assertEqual(site.name, 'A new Domain.')

    def test_new_domain_does_not_update_site_when_custom(self):
        # Test that when default Site isn't example.com, we don't change it.
        self.client.login(username='su', password='pwd')
        site = Site.objects.get(pk=1)
        site.domain = 'otherthan.example.com'
        site.name = 'Something other than Example.com'
        site.save()
        # Now, let's create a new Domain and check if the Site object is
        # udpated.
        post_data = {'mail_host': 'aexample.com',
                     'site': '1',
                     'description': 'A new Domain.'}
        response = self.client.post(reverse('domain_new'), post_data)
        self.assertEqual(response.status_code, 302)
        self.assertHasSuccessMessage(response)
        # Get the site  object from database.
        site = Site.objects.get(pk=1)
        self.assertEqual(site.domain, 'otherthan.example.com')
        self.assertEqual(site.name, 'Something other than Example.com')



# -*- coding: utf-8 -*-
# Copyright (C) 2016-2021 by the Free Software Foundation, Inc.
#
# This file is part of Postorius.
#
# Postorius is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option)
# any later version.
# Postorius is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along with
# Postorius.  If not, see <http://www.gnu.org/licenses/>.


from django.test import TestCase
from django.urls import NoReverseMatch, reverse


class URLTest(TestCase):

    def test_email_allows_slash(self):
        try:
            reverse('list_member_options', kwargs={
                    'list_id': 'test.example.com',
                    'email': 'slashed/are/allowed@example.com',
                    })
            reverse('remove_role', kwargs={
                    'list_id': 'test.example.com',
                    'role': 'subscriber',
                    'address': 'slashed/are/allowed@example.com',
                    })
        except NoReverseMatch as e:
            self.fail(e)

    def test_held_message_url_ends_with_slash(self):
        url = reverse('rest_held_message', args=('foo', 0))
        self.assertEqual(url[-2:], '0/')



# -*- coding: utf-8 -*-
# Copyright (C) 1998-2021 by the Free Software Foundation, Inc.
#
# This file is part of Postorius.
#
# Postorius is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option)
# any later version.
#
# Postorius is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along with
# Postorius.  If not, see <http://www.gnu.org/licenses/>.

"""
Django test settings for postorius project.
"""

from settings import *


TESTING = True

# Mailman API credentials for testing
MAILMAN_REST_API_URL = 'http://localhost:9001'
MAILMAN_REST_API_USER = 'restadmin'
MAILMAN_REST_API_PASS = 'restpass'


LOGGING = {
    'version': 1,
    'disable_existing_loggers': False,
    'handlers': {
        'console': {
            'class': 'logging.StreamHandler',
            'formatter': 'verbose',
        },
    },
    'loggers': {
        'django': {
            'handlers': ['console'],
            'level': 'INFO',
        },
        'django.request': {
            'handlers': ['console'],
            'level': 'ERROR',
        },
        'postorius': {
            'handlers': ['console'],
            'level': 'INFO',
        },
    },
    'formatters': {
        'verbose': {
            'format': '%(levelname)s %(asctime)s %(process)d %(name)s %(message)s'
        },
    },
}



# -*- coding: utf-8 -*-
# Copyright (C) 2015-2021 by the Free Software Foundation, Inc.
#
# This file is part of Postorius.
#
# Postorius is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option)
# any later version.
#
# Postorius is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along with
# Postorius.  If not, see <http://www.gnu.org/licenses/>.


import logging
from urllib.error import HTTPError

from mailmanclient import MailmanConnectionError

from postorius import utils
from postorius.models import MailmanApiError


logger = logging.getLogger('postorius')


__all__ = [
    'PostoriusMiddleware',
]


class PostoriusMiddleware(object):

    def __init__(self, get_response=None):
        self.get_response = get_response

    def __call__(self, request):
        return self.get_response(request)

    def process_exception(self, request, exception):
        if isinstance(exception, (MailmanApiError, MailmanConnectionError)):
            logger.exception('Mailman REST API not available')
            return utils.render_api_error(request)
        elif isinstance(exception, HTTPError):
            logger.exception('Un-handled exception: %s', str(exception))
            return utils.render_client_error(request, exception)



# -*- coding: utf-8 -*-
# Copyright (C) 1998-2021 by the Free Software Foundation, Inc.
#
# This file is part of Postorius.
#
# Postorius is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option)
# any later version.
#
# Postorius is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along with
# Postorius.  If not, see <http://www.gnu.org/licenses/>.


import logging
from urllib.error import HTTPError

from django.contrib import messages
from django.contrib.auth.decorators import login_required
from django.forms import formset_factory
from django.http import Http404
from django.shortcuts import render
from django.urls import reverse, reverse_lazy
from django.utils.decorators import method_decorator
from django.utils.translation import gettext as _
from django.views.generic import FormView

from allauth.account.models import EmailAddress
from django_mailman3.lib.mailman import get_mailman_client, get_mailman_user

from postorius.forms import (
    ChangeSubscriptionForm, UserPreferences, UserPreferencesFormset)
from postorius.models import List, MailmanUser, SubscriptionMode
from postorius.utils import set_preferred
from postorius.views.generic import MailmanClientMixin


logger = logging.getLogger(__name__)


class UserPreferencesView(FormView, MailmanClientMixin):
    """Generic view for the logged-in user's various preferences."""

    form_class = UserPreferences

    def get_context_data(self, **kwargs):
        data = super(UserPreferencesView, self).get_context_data(**kwargs)
        data['mm_user'] = self.mm_user
        return data

    def get_form_kwargs(self):
        kwargs = super(UserPreferencesView, self).get_form_kwargs()
        kwargs['preferences'] = self._get_preferences()
        return kwargs

    def _set_view_attributes(self, request, *args, **kwargs):
        self.mm_user = MailmanUser.objects.get_or_create_from_django(
            request.user)

    @method_decorator(login_required)
    def dispatch(self, request, *args, **kwargs):
        self._set_view_attributes(request, *args, **kwargs)
        return super(UserPreferencesView, self).dispatch(
            request, *args, **kwargs)

    def form_valid(self, form):
        try:
            form.save()
        except HTTPError as e:
            messages.error(self.request, e.msg)
        if form.has_changed():
            messages.success(
                self.request, _('Your preferences have been updated.'))
        else:
            messages.info(self.request, _('Your preferences did not change.'))
        return super(UserPreferencesView, self).form_valid(form)


class UserMailmanSettingsView(UserPreferencesView):
    """The logged-in user's global Mailman preferences."""

    form_class = UserPreferences
    template_name = 'postorius/user/mailman_settings.html'
    success_url = reverse_lazy('user_mailmansettings')

    def _get_preferences(self):
        # Get the defaults and pre-populate so view shows them
        combinedpreferences = self._get_combined_preferences()
        for key in combinedpreferences:
            if key != "self_link":
                self.mm_user.preferences[key] = combinedpreferences[key]

        # This is a bit of a hack so preferences behave as users expect
        # We probably don't want to save, only display here
        # but this means that whatever preferences the users see first are
        # the ones they have unless they explicitly change them
        self.mm_user.preferences.save()

        return self.mm_user.preferences

    def _get_combined_preferences(self):
        # Get layers of default preferences to match how they are applied
        # We ignore self_link as we don't want to over-write it
        defaultpreferences = get_mailman_client().preferences
        combinedpreferences = {}
        for key in defaultpreferences:
            if key != "self_link":
                combinedpreferences[key] = defaultpreferences[key]

        # Clobber defaults with any preferences already set
        for key in self.mm_user.preferences:
            if key != "self_link":
                combinedpreferences[key] = self.mm_user.preferences[key]

        return(combinedpreferences)


class UserAddressPreferencesView(UserPreferencesView):
    """The logged-in user's address-based Mailman Preferences."""

    template_name = 'postorius/user/address_preferences.html'
    success_url = reverse_lazy('user_address_preferences')

    def get_form_class(self):
        return formset_factory(
            UserPreferences, formset=UserPreferencesFormset, extra=0)

    def _get_preferences(self):
        return [address.preferences for address in self.mm_user.addresses]

    def _get_combined_preferences(self):
        # grab the default preferences
        defaultpreferences = get_mailman_client().preferences

        # grab your global preferences
        globalpreferences = self.mm_user.preferences

        # start a new combined preferences object
        combinedpreferences = []

        for address in self.mm_user.addresses:
            # make a per-address prefs object
            prefs = {}

            # initialize with default preferences
            for key in defaultpreferences:
                if key != "self_link":
                    prefs[key] = defaultpreferences[key]

            # overwrite with user's global preferences
            for key in globalpreferences:
                if key != "self_link":
                    prefs[key] = globalpreferences[key]

            # overwrite with address-specific preferences
            for key in address.preferences:
                if key != "self_link":
                    prefs[key] = address.preferences[key]
            combinedpreferences.append(prefs)

            # put the combined preferences back on the original object
            for key in prefs:
                if key != "self_link":
                    address.preferences[key] = prefs[key]

        return combinedpreferences

    def get_context_data(self, **kwargs):
        data = super(UserAddressPreferencesView, self).get_context_data(
            **kwargs)
        data['formset'] = data.pop('form')
        for form, address in list(zip(
                data['formset'].forms, self.mm_user.addresses)):
            form.address = address
        return data


class UserListOptionsView(UserPreferencesView):
    """The logged-in user's subscription preferences."""

    form_class = UserPreferences
    template_name = 'postorius/user/list_options.html'

    def _get_subscription(self):
        subscription = None
        for s in self.mm_user.subscriptions:
            if s.role == 'member' and s.list_id == self.mlist.list_id:
                subscription = s
                break
        if not subscription:
            raise Http404(_('Subscription does not exist'))
        return subscription

    def _set_view_attributes(self, request, *args, **kwargs):
        super(UserListOptionsView, self)._set_view_attributes(
            request, *args, **kwargs)
        self.mlist = List.objects.get_or_404(fqdn_listname=kwargs['list_id'])
        self.subscription = self._get_subscription()
        if (self.subscription.subscription_mode ==
                SubscriptionMode.as_user.name):
            self.subscriber = self.subscription.user.user_id
        else:
            self.subscriber = self.subscription.email

    def _get_preferences(self):
        return self.subscription.preferences

    def get_context_data(self, **kwargs):
        data = super(UserListOptionsView, self).get_context_data(**kwargs)
        data['mlist'] = self.mlist
        user_emails = EmailAddress.objects.filter(
            user=self.request.user, verified=True).order_by(
            "email").values_list("email", flat=True)
        mm_user = get_mailman_user(self.request.user)
        primary_email = None
        if mm_user.preferred_address is None:
            primary_email = set_preferred(self.request.user, mm_user)
        else:
            primary_email = mm_user.preferred_address.email
        data['change_subscription_form'] = ChangeSubscriptionForm(
            user_emails, mm_user.user_id, primary_email,
            initial={'subscriber': self.subscriber})
        return data

    def get_success_url(self):
        return reverse(
            'user_list_options', kwargs=dict(list_id=self.mlist.list_id))


class UserSubscriptionPreferencesView(UserPreferencesView):
    """The logged-in user's subscription-based Mailman Preferences."""

    template_name = 'postorius/user/subscription_preferences.html'
    success_url = reverse_lazy('user_subscription_preferences')

    def _get_subscriptions(self):
        subscriptions = []
        for s in self.mm_user.subscriptions:
            if s.role != 'member':
                continue
            subscriptions.append(s)
        return subscriptions

    def _set_view_attributes(self, request, *args, **kwargs):
        super(UserSubscriptionPreferencesView, self)._set_view_attributes(
            request, *args, **kwargs)
        self.subscriptions = self._get_subscriptions()

    def get_form_class(self):
        return formset_factory(
            UserPreferences, formset=UserPreferencesFormset, extra=0)

    def _get_preferences(self):
        return [sub.preferences for sub in self.subscriptions]

    def _get_combined_preferences(self):
        # grab the default preferences
        defaultpreferences = get_mailman_client().preferences

        # grab your global preferences
        globalpreferences = self.mm_user.preferences

        # start a new combined preferences object
        combinedpreferences = []

        for sub in self.subscriptions:
            # make a per-address prefs object
            prefs = {}

            # initialize with default preferences
            for key in defaultpreferences:
                if key != "self_link":
                    prefs[key] = defaultpreferences[key]

            # overwrite with user's global preferences
            for key in globalpreferences:
                if key != "self_link":
                    prefs[key] = globalpreferences[key]

            # overwrite with address-based preferences
            # There is currently no better way to do this,
            # we may consider revisiting.
            addresspreferences = {}
            for address in self.mm_user.addresses:
                if sub.email == address.email:
                    addresspreferences = address.preferences

            for key in addresspreferences:
                if key != "self_link":
                    prefs[key] = addresspreferences[key]

            # overwrite with subscription-specific preferences
            for key in sub.preferences:
                if key != "self_link":
                    prefs[key] = sub.preferences[key]

            combinedpreferences.append(prefs)

        return combinedpreferences
        # return [sub.preferences for sub in self.subscriptions]

    def get_context_data(self, **kwargs):
        data = super(UserSubscriptionPreferencesView, self).get_context_data(
            **kwargs)
        data['formset'] = data.pop('form')
        for form, subscription in list(zip(
                data['formset'].forms, self.subscriptions)):
            form.list_id = subscription.list_id
        return data


@login_required
def user_subscriptions(request):
    """Shows the subscriptions of a user."""
    mm_user = MailmanUser.objects.get_or_create_from_django(request.user)
    memberships = [m for m in mm_user.subscriptions]
    return render(request, 'postorius/user/subscriptions.html',
                  {'memberships': memberships})



# Copyright (C) 2012-2021 by the Free Software Foundation, Inc.
#
# This file is part of Postorius.
#
# Postorius is free software: you can redistribute it and/or modify it
# under the terms of the GNU Lesser General Public License as published by the
# Free Software Foundation, either version 3 of the License, or (at your
# option) any later version.
#
# Postorius is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License
# for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with Postorius.  If not, see <http://www.gnu.org/licenses/>.

import re
import sys

from setuptools import find_packages, setup


# Calculate the version number without importing the postorius package.
with open('src/postorius/__init__.py') as fp:
    for line in fp:
        mo = re.match("__version__ = '(?P<version>[^']+?)'", line)
        if mo:
            __version__ = mo.group('version')
            break
    else:
        print('No version number found')
        sys.exit(1)


setup(
    name="postorius",
    version=__version__,
    description="A web user interface for GNU Mailman",
    long_description=open('README.rst').read(),
    maintainer="The Mailman GSOC Coders",
    license='GPLv3',
    keywords='email mailman django',
    url=" https://gitlab.com/mailman/postorius",
    classifiers=[
        "Framework :: Django",
        "Development Status :: 4 - Beta",
        "License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
        "Topic :: Communications :: Email :: Mailing List Servers",
        "Programming Language :: Python :: 3",
    ],
    packages=find_packages('src'),
    package_dir={'': 'src'},
    include_package_data=True,
    install_requires=[
        'django-mailman3>=1.3.5',
        'Django>=1.11,<3.2',
        'mailmanclient>=3.3.2',
        'readme_renderer[md]',
    ],
    tests_require=[
        "beautifulsoup4",
        "isort",
    ],
)



# -*- coding: utf-8 -*-
# Copyright (C) 2012-2021 by the Free Software Foundation, Inc.
#
# This file is part of Postorius.
#
# Postorius is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option)
# any later version.
# Postorius is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along with
# Postorius.  If not, see <http://www.gnu.org/licenses/>.

import os
from unittest.mock import MagicMock

from django.conf import settings
from django.contrib import messages
from django.test import TransactionTestCase
from django.urls import reverse

from django_mailman3.lib.mailman import get_mailman_client
from django_mailman3.tests.utils import get_flash_messages
from six import PY3, binary_type, text_type
from six.moves.urllib_parse import (
    parse_qsl, quote, urlencode, urlparse, urlunparse)


def get_test_file(*fileparts):
    return os.path.join(os.path.dirname(__file__), "test_data", *fileparts)
get_test_file.__test__ = False  # noqa: E305


def reorder_request_params(request):
    def reorder_params(params):
        parsed = None
        if PY3:
            if isinstance(params, binary_type):
                params = params.decode("ascii")
            parsed = parse_qsl(params, encoding="utf-8")
        else:
            parsed = parse_qsl(params)
        if parsed:
            return urlencode(sorted(parsed, key=lambda kv: kv[0]))
        else:
            # Parsing failed, it may be a simple string.
            return params
        # sort the URL query-string by key names.
    uri_parts = urlparse(request.uri)
    if uri_parts.query:
        request.uri = urlunparse((
            uri_parts.scheme, uri_parts.netloc, uri_parts.path,
            uri_parts.params, reorder_params(uri_parts.query),
            uri_parts.fragment,
        ))
        # convert the request body to text and sort the parameters.
    if isinstance(request.body, binary_type):
        try:
            request._body = request._body.decode('utf-8')
        except UnicodeDecodeError:
            pass
    if isinstance(request.body, text_type):
        request._body = reorder_params(request._body.encode('utf-8'))
    return request


def filter_response_headers(response):
    for header in ('Date', 'Server', 'date', 'server'):
        # The headers are lowercase on Python 2 and capitalized on Python 3
        if header in response['headers']:
            del response['headers'][header]
    return response


def create_mock_domain(properties=None):
    """Create and return a mocked Domain.

    :param properties: A dictionary of the domain's properties.
    :type properties: dict
    :return: A MagicMock object with the properties set.
    :rtype: MagicMock
    """
    mock_object = MagicMock(name='Domain')
    mock_object.contact_address = ''
    mock_object.description = ''
    mock_object.mail_host = ''
    mock_object.lists = []
    if properties is not None:
        for key in properties:
            setattr(mock_object, key, properties[key])
    return mock_object


def create_mock_list(properties=None):
    """Create and return a mocked List.

    :param properties: A dictionary of the list's properties.
    :type properties: dict
    :return: A MagicMock object with the properties set.
    :rtype: MagicMock
    """
    mock_object = MagicMock(name='List')
    mock_object.members = []
    mock_object.moderators = []
    mock_object.owners = []
    # like in mock_domain, some defaults need to be added...
    if properties is not None:
        for key in properties:
            setattr(mock_object, key, properties[key])
    return mock_object


def create_mock_member(properties=None):
    """Create and return a mocked Member.

    :param properties: A dictionary of the member's properties.
    :type properties: dict
    :return: A MagicMock object with the properties set.
    :rtype: MagicMock
    """
    mock_object = MagicMock(name='Member')
    # like in mock_domain, some defaults need to be added...
    if properties is not None:
        for key in properties:
            setattr(mock_object, key, properties[key])
    return mock_object


class ViewTestCase(TransactionTestCase):

    def setUp(self):
        self.mm_client = get_mailman_client()

    def tearDown(self):
        for d in self.mm_client.domains:
            d.delete()
        for u in self.mm_client.users:
            u.delete()

    def assertHasSuccessMessage(self, response, count=1):
        msgs = get_flash_messages(response)
        self.assertEqual(len(msgs), count)
        self.assertEqual(
            msgs[0].level, messages.SUCCESS,
            "%s: %s" % (messages.DEFAULT_TAGS[msgs[0].level], msgs[0].message))
        return msgs[0].message

    def assertHasErrorMessage(self, response):
        msgs = get_flash_messages(response)
        self.assertEqual(len(msgs), 1)
        self.assertEqual(
            msgs[0].level, messages.ERROR,
            "%s: %s" % (messages.DEFAULT_TAGS[msgs[0].level], msgs[0].message))
        return msgs[0].message

    def assertHasNoMessage(self, response):
        msgs = get_flash_messages(response)
        self.assertEqual(len(msgs), 0)

    def assertRedirectsToLogin(self, url):
        response = self.client.get(url)
        self.assertRedirects(response, '{}?next={}'.format(
            reverse(settings.LOGIN_URL), quote(url)))



# -*- coding: utf-8 -*-
# Copyright (C) 2012-2021 by the Free Software Foundation, Inc.
#
# This file is part of Postorius.
#
# Postorius is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option)
# any later version.
# Postorius is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along with
# Postorius.  If not, see <http://www.gnu.org/licenses/>.

from urllib.error import HTTPError

from django.contrib.auth.models import User
from django.urls import reverse

from allauth.account.models import EmailAddress
from django_mailman3.models import MailDomain

from postorius.tests.utils import ViewTestCase


class DomainIndexPageTest(ViewTestCase):
    """Tests for the list index page."""

    def setUp(self):
        super(DomainIndexPageTest, self).setUp()
        self.domain = self.mm_client.create_domain('example.com')
        self.domain.add_owner('person@domain.com')
        try:
            self.foo_list = self.domain.create_list('foo')
        except HTTPError:
            self.foo_list = self.mm_client.get_list('foo.example.com')

        self.user = User.objects.create_user(
            'testuser', 'test@example.com', 'testpass')
        self.superuser = User.objects.create_superuser(
            'testsu', 'su@example.com', 'testpass')
        self.owner = User.objects.create_user(
            'testowner', 'owner@example.com', 'testpass')
        self.moderator = User.objects.create_user(
            'testmoderator', 'moderator@example.com', 'testpass')
        for user in (self.user, self.superuser, self.owner, self.moderator):
            EmailAddress.objects.create(
                user=user, email=user.email, verified=True)
        self.foo_list.add_owner('owner@example.com')
        self.foo_list.add_moderator('moderator@example.com')

    def _test_not_accesible_to_public(self, url):
        response = self.client.get(url)
        self.assertEqual(response.status_code, 302)

    def _test_not_accessible_to_unpriveleged_use(self, url):
        self.client.login(username='testuser', password='testpass')
        response = self.client.get(url)
        self.assertEqual(response.status_code, 403)

    def _test_not_accessible_to_moderators(self, url):
        self.client.login(username='testmoderator', password='testpass')
        response = self.client.get(url)
        self.assertEqual(response.status_code, 403)

    def _test_not_accessible_to_owner(self, url):
        self.client.login(username='testowner', password='testpass')
        response = self.client.get(url)
        self.assertEqual(response.status_code, 403)

    def test_domain_index_not_accessible_to_public(self):
        self._test_not_accesible_to_public(reverse('domain_index'))

    def test_domain_index_not_accessible_to_unpriveleged_user(self):
        self._test_not_accessible_to_unpriveleged_use(reverse('domain_index'))

    def test_domain_index_not_accessible_to_moderators(self):
        self._test_not_accessible_to_moderators(reverse('domain_index'))

    def test_domain_index_not_accessible_to_owners(self):
        self._test_not_accessible_to_owner(reverse('domain_index'))

    def test_contains_domains_and_site(self):
        # The list index page should contain the lists
        self.client.login(username='testsu', password='testpass')
        response = self.client.get(reverse('domain_index'))
        self.assertEqual(response.status_code, 200)
        self.assertEqual(len(response.context['domains']), 1)
        self.assertContains(response, 'example.com')
        self.assertTrue(
            MailDomain.objects.filter(mail_domain='example.com').exists())
        # Test there are owners are listed.
        self.assertContains(response, 'person@domain.com')

    def test_domain_add_owner_not_acceesible_to_anyone_but_superuser(self):
        url = reverse('domain_owners', args=(self.domain.mail_host,))
        self._test_not_accesible_to_public(url)
        self._test_not_accessible_to_unpriveleged_use(url)
        self._test_not_accessible_to_moderators(url)
        self._test_not_accessible_to_owner(url)
        self.client.login(username='testsu', password='testpass')
        response = self.client.get(url)
        self.assertEqual(response.status_code, 200)
        self.assertContains(response, b"Add a new owner to example.com")
        response = self.client.post(
            url,
            dict(email='person@example.com'))
        self.assertEqual(response.status_code, 302)
        self.assertIn(
            'person@example.com',
            [owner.addresses[0].email for owner in self.domain.owners])

    def test_domain_delete_owner_not_acceesible_to_anyone_but_superuser(self):
        self.domain.add_owner('one@example.com')
        self.domain.add_owner('two@example.com')
        url = reverse('remove_domain_owner',
                      args=(self.domain.mail_host,
                            'person@domain.com'))
        self.client.login(username='testsu', password='testpass')
        response = self.client.post(url)
        self.assertEqual(response.status_code, 302)
        self.assertTrue(len(self.domain.owners), 2)
        self.assertEqual(sorted(owner.addresses[0].email
                                for owner in self.domain.owners),
                         ['one@example.com', 'two@example.com'])



# -*- coding: utf-8 -*-
# Copyright (C) 2012-2021 by the Free Software Foundation, Inc.
#
# This file is part of Postorius.
#
# Postorius is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option)
# any later version.
# Postorius is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along with
# Postorius.  If not, see <http://www.gnu.org/licenses/>.


from django.contrib.auth.models import User
from django.urls import reverse

from allauth.account.models import EmailAddress
from django_mailman3.lib.mailman import get_mailman_user

from postorius.forms import ListAnonymousSubscribe
from postorius.tests.utils import ViewTestCase


class ListSummaryPageTest(ViewTestCase):
    """Tests for the list summary page.

    Tests accessiblity and existince of the submit form depending on
    login status.
    """

    def setUp(self):
        super(ListSummaryPageTest, self).setUp()
        self.domain = self.mm_client.create_domain('example.com')
        self.foo_list = self.domain.create_list('foo')
        self.user = User.objects.create_user(
            'testuser', 'test@example.com', 'testpass')
        EmailAddress.objects.create(
            user=self.user, email=self.user.email, verified=True, primary=True)

    def test_list_summary_logged_out(self):
        # Response must contain list obj and anonymous subscribe form.
        response = self.client.get(reverse('list_summary',
                                   args=('foo@example.com', )))
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.context['list'].fqdn_listname,
                         'foo@example.com')
        self.assertIsInstance(response.context['anonymous_subscription_form'],
                              ListAnonymousSubscribe)
        self.assertContains(response, '<form ')

    def test_list_summary_logged_in(self):
        # Response must contain list obj and the form.
        self.client.login(username='testuser', password='testpass')
        response = self.client.get(reverse('list_summary',
                                   args=('foo@example.com', )))
        self.assertEqual(response.status_code, 200)
        self.assertContains(response, '<form ')
        self.assertContains(response, 'Subscribe')

    def test_pending_subscription_request(self):
        mlist = self.mm_client.get_list('foo@example.com')
        mlist.settings['subscription_policy'] = 'moderate'
        mlist.settings.save()
        mlist.subscribe('test@example.com',
                        pre_verified=True,
                        pre_confirmed=True)
        self.client.login(username='testuser', password='testpass')
        response = self.client.get(reverse('list_summary',
                                           args=('foo@example.com', )))
        self.assertEqual(response.status_code, 200)
        self.assertContains(response, 'You have a subscription request '
                                      'pending. If you don\'t hear back soon, '
                                      'please contact the list owners.')
        self.assertNotContains(response, 'Unsubscribe')
        self.assertNotContains(response, 'Subscribe')

    def test_unsubscribe_button_is_available(self):
        mlist = self.mm_client.get_list('foo@example.com')
        mlist.subscribe('test@example.com',
                        pre_verified=True,
                        pre_confirmed=True)
        self.client.login(username='testuser', password='testpass')
        response = self.client.get(reverse('list_summary',
                                           args=('foo@example.com', )))
        self.assertEqual(response.status_code, 200)
        self.assertContains(response, 'Unsubscribe')

    def test_list_summary_owner(self):
        # Response must contain the administration menu
        self.mm_client.create_user('test@example.com', None)
        mlist = self.mm_client.get_list('foo@example.com')
        mlist.add_owner('test@example.com')
        self.client.login(username='testuser', password='testpass')
        response = self.client.get(reverse('list_summary',
                                   args=('foo@example.com', )))
        self.assertContains(response, 'Delete</a>')

    def test_list_summary_moderator(self):
        # Response must contain the administration menu
        self.mm_client.create_user('test@example.com', None)
        mlist = self.mm_client.get_list('foo@example.com')
        mlist.add_moderator('test@example.com')
        self.client.login(username='testuser', password='testpass')
        response = self.client.get(reverse('list_summary',
                                   args=('foo@example.com', )))
        self.assertContains(response, 'Held messages')
        self.assertNotContains(response, 'Delete</a>')

    def test_list_summary_is_admin_secondary_owner(self):
        # Response must contain the administration menu
        EmailAddress.objects.create(
            user=self.user, email='anotheremail@example.com', verified=True)
        user = self.mm_client.create_user('test@example.com', None)
        address = user.add_address('anotheremail@example.com')
        address.verify()
        mlist = self.mm_client.get_list('foo@example.com')
        mlist.add_owner('anotheremail@example.com')
        self.client.login(username='testuser', password='testpass')
        response = self.client.get(reverse('list_summary',
                                   args=('foo@example.com', )))
        self.assertContains(response, 'Delete</a>')

    def test_list_summary_is_admin_secondary_moderator(self):
        # Response must contain the administration menu
        EmailAddress.objects.create(
            user=self.user, email='anotheremail@example.com', verified=True)
        user = self.mm_client.create_user('test@example.com', None)
        address = user.add_address('anotheremail@example.com')
        address.verify()
        mlist = self.mm_client.get_list('foo@example.com')
        mlist.add_moderator('anotheremail@example.com')
        self.client.login(username='testuser', password='testpass')
        response = self.client.get(reverse('list_summary',
                                   args=('foo@example.com', )))
        self.assertEqual(response.status_code, 200)
        self.assertContains(response, 'Held messages')
        self.assertNotContains(response, 'Delete</a>')

    def test_metrics_not_displayed_to_anonymous(self):
        response = self.client.get(reverse('list_summary',
                                           args=('foo@example.com',)))
        self.assertNotContains(response, 'List metrics')

    def test_list_metrics_not_displayed_to_moderator(self):
        mlist = self.mm_client.get_list('foo@example.com')
        mlist.add_moderator('test@example.com')
        self.client.login(username='testuser', password='testpass')
        response = self.client.get(reverse('list_summary',
                                           args=('foo@example.com',)))
        self.assertNotContains(response, 'List metrics')

    def test_list_metrics_displayed_to_owner(self):
        mlist = self.mm_client.get_list('foo@example.com')
        mlist.add_owner('test@example.com')
        self.client.login(username='testuser', password='testpass')
        response = self.client.get(reverse('list_summary',
                                           args=('foo@example.com',)))
        self.assertContains(response, 'List metrics')

    def test_list_metrics_displayed_to_superuser(self):
        user = User.objects.create_superuser(
            'testadminuser', 'testadmin@example.com', 'testpass')
        EmailAddress.objects.create(
            user=user, email=user.email, verified=True)
        self.assertTrue(self.client.login(username='testadminuser',
                                          password='testpass'))
        response = self.client.get(reverse('list_summary',
                                           args=('foo@example.com',)))
        self.assertContains(response, 'List metrics')

    def test_list_info(self):
        # Test that list info is rendered as markdown.
        settings = self.mm_client.get_list('foo@example.com').settings
        info = 'Welcome To FooList today. This is something very interesting.'
        settings['info'] = info
        settings.save()
        response = self.client.get(reverse('list_summary',
                                           args=('foo@example.com',)))
        self.assertContains(response, info)
        settings['info'] = """\
Welcome To Foolist
==================

```
def function:
    print('Hello World')
```
"""
        settings.save()
        response = self.client.get(reverse('list_summary',
                                           args=('foo@example.com',)))
        self.assertContains(response, '<pre><code>def function:')

    def test_list_summary_already_subscribed(self):
        self.foo_list.subscribe(
            'test@example.com',
            pre_verified=True, pre_confirmed=True, pre_approved=True)
        self.client.login(username='testuser', password='testpass')
        response = self.client.get(reverse('list_summary',
                                           args=('foo@example.com', )))
        # Assert two parts separately due to sometimes being newlines and stuff
        # in the response HTML.
        self.assertContains(response,
                            'You are subscribed to this list with the '
                            'following address:')
        self.assertContains(response, ' <em>test@example.com</em>')

    def test_list_summary_already_subscribed_user(self):
        mm_user = get_mailman_user(self.user)
        mm_user.addresses[0].verify()
        mm_user.preferred_address = self.user.email
        self.foo_list.subscribe(
            str(mm_user.user_id),
            pre_verified=True, pre_confirmed=True, pre_approved=True)
        self.client.login(username='testuser', password='testpass')
        response = self.client.get(reverse('list_summary',
                                           args=('foo@example.com', )))
        # Assert two parts separately due to sometimes being newlines and stuff
        # in the response HTML.
        self.assertContains(response,
                            'You are subscribed to this list with the '
                            'following address:')
        self.assertContains(response,
                            ' <em>Primary Address (test@example.com)</em>')

    def test_list_summary_sets_preferred_address(self):
        # Test that preferred address is set.
        mm_user = get_mailman_user(self.user)
        self.assertIsNone(mm_user.preferred_address)
        self.client.login(username='testuser', password='testpass')
        self.client.get(reverse('list_summary',
                                args=('foo@example.com', )))
        self.assertEqual(mm_user.preferred_address.email, self.user.email)

    def test_list_summary_not_verified_does_not_set_preferred(self):
        # Test that the preferred address is *not* set if the primary_email is
        # not verified.
        primary_email = EmailAddress.objects.create(
            user=self.user, email='another@example.com', verified=False)
        primary_email.set_as_primary()
        mm_user = get_mailman_user(self.user)
        self.assertIsNone(mm_user.preferred_address)
        self.client.get(reverse('list_summary',
                                args=('foo@example.com', )))
        self.assertIsNone(mm_user.preferred_address)



# -*- coding: utf-8 -*-
# Copyright (C) 2012-2021 by the Free Software Foundation, Inc.
#
# This file is part of Postorius.
#
# Postorius is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option)
# any later version.
# Postorius is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along with
# Postorius.  If not, see <http://www.gnu.org/licenses/>.


from django.conf import settings
from django.contrib.auth.models import User
from django.urls import reverse

from allauth.account.models import EmailAddress
from django_mailman3.lib.mailman import get_mailman_client
from six.moves.urllib_parse import quote

from postorius.forms import MemberModeration, UserPreferences
from postorius.tests.utils import ViewTestCase


class ListMembersOptionsTest(ViewTestCase):
    """Tests for the list members page.

    Tests permissions and creation of list owners and moderators.
    """

    def setUp(self):
        super(ListMembersOptionsTest, self).setUp()
        self.domain = get_mailman_client().create_domain('example.com')
        self.foo_list = self.domain.create_list('foo')
        self.user = User.objects.create_user(
            'testuser', 'test@example.com', 'testpass')
        self.superuser = User.objects.create_superuser(
            'testsu', 'su@example.com', 'testpass')
        self.owner = User.objects.create_user(
            'testowner', 'owner@example.com', 'testpass')
        self.moderator = User.objects.create_user(
            'testmoderator', 'moderator@example.com', 'testpass')
        for user in (self.user, self.superuser, self.owner, self.moderator):
            EmailAddress.objects.create(
                user=user, email=user.email, verified=True)
        self.foo_list.add_owner('owner@example.com')
        self.foo_list.add_moderator('moderator@example.com')
        self.mm_user = get_mailman_client().create_user('test@example.com', '')
        self.mm_user.addresses[0].verify()
        self.foo_list.subscribe('test@example.com', pre_verified=True,
                                pre_confirmed=True, pre_approved=True)
        self.url = reverse('list_member_options', args=(self.foo_list.list_id,
                                                        'test@example.com',))

    def test_page_not_accessible_if_not_logged_in(self):
        response = self.client.get(self.url)
        self.assertRedirects(response, '{}?next={}'.format(
            reverse(settings.LOGIN_URL), quote(self.url)))

    def test_page_not_accessible_for_unprivileged_users(self):
        self.client.login(username='testuser', password='testpass')
        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 403)

    def test_not_accessible_for_moderator(self):
        self.client.login(username='testmoderator', password='testpass')
        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 403)

    def test_page_accessible_for_superuser(self):
        self.client.login(username='testsu', password='testpass')
        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 200)
        self.assertIsInstance(response.context['moderation_form'],
                              MemberModeration)
        self.assertIsInstance(response.context['preferences_form'],
                              UserPreferences)

    def test_page_accessible_for_owner(self):
        self.client.login(username='testowner', password='testpass')
        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 200)
        self.assertIsInstance(response.context['moderation_form'],
                              MemberModeration)
        self.assertIsInstance(response.context['preferences_form'],
                              UserPreferences)

    def test_nonexistent_member_returns_404(self):
        self.client.login(username='testsu', password='testpass')
        url = reverse('list_member_options', args=(self.foo_list.list_id,
                                                   'none@example.com',))
        response = self.client.get(url)
        self.assertEqual(response.status_code, 404)

    def test_moderation_action(self):
        self.assertIsNone(
            self.foo_list.get_member('test@example.com').moderation_action)
        self.client.login(username='testsu', password='testpass')
        response = self.client.post(self.url, {
            'formname': 'moderation', 'moderation_action': 'hold'})
        self.assertRedirects(response, self.url)
        self.assertHasSuccessMessage(response)
        self.assertEqual(
            self.foo_list.get_member('test@example.com').moderation_action,
            'hold')
        response = self.client.post(self.url, {
            'formname': 'moderation', 'moderation_action': ''})
        self.assertRedirects(response, self.url)
        self.assertHasSuccessMessage(response)
        self.assertIsNone(
            self.foo_list.get_member('test@example.com').moderation_action)

    def test_get_nonmember_options(self):
        # Test that nonmember options can also be set using
        # `list_member_options` view.
        self.client.login(username='testowner', password='testpass')
        self.foo_list.add_role(
            role='nonmember', address='nonmember@example.com')
        self.assertEqual(len(self.foo_list.nonmembers), 1)
        # Now, let's try to get options for these users.
        url = reverse('list_member_options', args=(self.foo_list.list_id,
                                                   'nonmember@example.com',))
        response = self.client.get(url)
        self.assertEqual(response.status_code, 200)

    def test_nonmember_moderation_action(self):
        self.foo_list.add_role(
            role='nonmember', address='nonmember@example.com')
        nonmember = self.foo_list.find_members(
            address='nonmember@example.com')[0]
        self.assertIsNone(nonmember.moderation_action)
        url = reverse('list_member_options', args=(self.foo_list.list_id,
                                                   'nonmember@example.com',))
        self.client.login(username='testsu', password='testpass')
        response = self.client.post(url, {
            'formname': 'moderation', 'moderation_action': 'hold'})
        self.assertRedirects(response, url)
        self.assertHasSuccessMessage(response)
        nonmember = self.foo_list.find_members(
            address='nonmember@example.com')[0]
        self.assertEqual(nonmember.moderation_action, 'hold')
        response = self.client.post(url, {
            'formname': 'moderation', 'moderation_action': ''})
        self.assertRedirects(response, url)
        self.assertHasSuccessMessage(response)
        nonmember = self.foo_list.find_members(
            address='nonmember@example.com')[0]
        self.assertIsNone(nonmember.moderation_action)

    def test_nonmember_and_owner_with_same_email_action(self):
        # Test that when an email address with both owner and non-member roles
        # are subscribed that we are able to moderate the non-member using the
        # list options view.
        self.foo_list.add_role(
            role='nonmember', address='aperson@example.com')
        self.foo_list.add_role(
            role='owner', address='aperson@example.com')
        url = reverse('list_member_options',
                      args=(self.foo_list.list_id,
                            'aperson@example.com',)) + '?role=nonmember'
        self.client.login(username='testsu', password='testpass')
        response = self.client.get(url)
        member = response.context.get('mm_member')
        # Assert that the list options are rendered for the non-member object
        # and not the owner object.
        self.assertEqual(member.role, 'nonmember')
        self.assertEqual(member.email, 'aperson@example.com')



# -*- coding: utf-8 -*-
# Copyright (C) 2012-2021 by the Free Software Foundation, Inc.
#
# This file is part of Postorius.
#
# Postorius is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option)
# any later version.
# Postorius is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along with
# Postorius.  If not, see <http://www.gnu.org/licenses/>.


from django.contrib.auth.models import User
from django.urls import reverse

from allauth.account.models import EmailAddress

from postorius.tests.utils import ViewTestCase


class ListMembersAccessTest(ViewTestCase):
    """Tests for the list members page.

    Tests permissions and creation of list owners and moderators.
    """

    def setUp(self):
        super(ListMembersAccessTest, self).setUp()
        self.domain = self.mm_client.create_domain('example.com')
        self.foo_list = self.domain.create_list('foo')
        self.user = User.objects.create_user(
            'testuser', 'test@example.com', 'testpass')
        self.superuser = User.objects.create_superuser(
            'testsu', 'su@example.com', 'testpass')
        self.owner = User.objects.create_user(
            'testowner', 'owner@example.com', 'testpass')
        self.moderator = User.objects.create_user(
            'testmoderator', 'moderator@example.com', 'testpass')
        for user in (self.user, self.superuser, self.owner, self.moderator):
            EmailAddress.objects.create(
                user=user, email=user.email, verified=True)
        self.foo_list.add_owner('owner@example.com')
        self.foo_list.add_moderator('moderator@example.com')

    def test_page_not_accessible_if_not_logged_in(self):
        for role in ['owner', 'moderator', 'member', 'nonmember']:
            response = self.client.get(reverse(
                'list_members', args=('foo.example.com', role,)))
            self.assertEqual(response.status_code, 403)

    def test_page_not_accessible_for_unprivileged_users(self):
        self.client.login(username='testuser', password='testpass')
        for role in ['owner', 'moderator', 'member', 'nonmember']:
            response = self.client.get(reverse(
                'list_members', args=('foo.example.com', role,)))
            self.assertEqual(response.status_code, 403)

    def test_not_accessible_for_moderator(self):
        self.client.login(username='testmoderator', password='testpass')
        response = self.client.get(reverse(
            'list_members', args=('foo.example.com', 'member',)))
        self.assertEqual(response.status_code, 403)

    def test_page_accessible_for_superuser(self):
        self.client.login(username='testsu', password='testpass')
        response = self.client.get(reverse(
            'list_members', args=('foo.example.com', 'member',)))
        self.assertEqual(response.status_code, 200)

    def test_page_accessible_for_owner(self):
        self.client.login(username='testowner', password='testpass')
        response = self.client.get(reverse(
            'list_members', args=('foo.example.com', 'member',)))
        self.assertEqual(response.status_code, 200)


class AddRemoveOwnerTest(ViewTestCase):
    """Tests for the list members page.

    Tests creation of list owners.
    """

    def setUp(self):
        super(AddRemoveOwnerTest, self).setUp()
        self.mm_client = self.mm_client
        self.domain = self.mm_client.create_domain('example.com')
        self.foo_list = self.domain.create_list('foo')
        self.su = User.objects.create_superuser(
            'su', 'su@example.com', 'pwd')
        EmailAddress.objects.create(
            user=self.su, email=self.su.email, verified=True)
        self.client.login(username='su', password='pwd')
        self.mm_client.get_list('foo.example.com').add_owner('su@example.com')

    def test_add_remove_owner(self):
        url = reverse('list_members', args=('foo.example.com', 'owner',))
        response = self.client.post(url, {'email': 'newowner@example.com'})
        self.assertRedirects(response, url)
        owners_emails = [owner.email for owner in self.foo_list.owners]
        self.assertTrue('newowner@example.com' in owners_emails)
        self.client.post(
            reverse('remove_role', args=('foo.example.com', 'owner',
                                         'newowner@example.com')))
        owners_emails = [owner.email for owner in self.foo_list.owners]
        self.assertFalse('newowner@example.com' in owners_emails)

    def test_remove_owner_by_owner(self):
        owners_emails = [owner.email for owner in self.foo_list.owners]
        self.assertTrue('su@example.com' in owners_emails)
        # Make the logged in user a simple list owner
        self.su.is_superuser = False
        self.su.save()
        # It must still be allowed to create and remove owners
        url = reverse('list_members', args=('foo.example.com', 'owner',))
        response = self.client.post(url, {'email': 'newowner@example.com'})
        self.assertRedirects(response, url)
        owners_emails = [owner.email for owner in self.foo_list.owners]
        self.assertTrue('newowner@example.com' in owners_emails)
        response = self.client.post(
            reverse('remove_role', args=('foo.example.com', 'owner',
                                         'newowner@example.com')))
        self.assertHasSuccessMessage(response)
        owners_emails = [owner.email for owner in self.foo_list.owners]
        self.assertFalse('newowner@example.com' in owners_emails)

    def test_remove_owner_as_owner_self_last(self):
        # It is allowed to remove itself, but only if there's another owner
        # left.
        mm_list = self.mm_client.get_list('foo.example.com')
        mm_list.add_owner('otherowner@example.com')
        owners_emails = [owner.email for owner in mm_list.owners]
        self.assertTrue('su@example.com' in owners_emails)
        self.assertTrue('otherowner@example.com' in owners_emails)
        response = self.client.post(
            reverse('remove_role', args=('foo.example.com', 'owner',
                                         'su@example.com')))
        self.assertEqual(response.status_code, 302)
        self.assertHasSuccessMessage(response)
        owners_emails = [owner.email for owner in mm_list.owners]
        self.assertFalse('su@example.com' in owners_emails)
        # But not to remove the last owner
        mm_list.add_owner('su@example.com')
        mm_list.remove_owner('otherowner@example.com')
        owners_emails = [owner.email for owner in mm_list.owners]
        self.assertTrue('su@example.com' in owners_emails)
        self.assertFalse('otherowner@example.com' in owners_emails)
        response = self.client.post(
            reverse('remove_role', args=('foo.example.com', 'owner',
                                         'su@example.com')))
        self.assertEqual(response.status_code, 302)
        self.assertHasErrorMessage(response)
        owners_emails = [owner.email for owner in mm_list.owners]
        self.assertTrue('su@example.com' in owners_emails)


class AddModeratorTest(ViewTestCase):
    """Tests for the list members page.

    Tests creation of moderators.
    """

    def setUp(self):
        super(AddModeratorTest, self).setUp()
        self.domain = self.mm_client.create_domain('example.com')
        self.foo_list = self.domain.create_list('foo')
        self.su = User.objects.create_superuser(
            'su', 'su@example.com', 'pwd')
        EmailAddress.objects.create(
            user=self.su, email=self.su.email, verified=True)
        # login and post new moderator data to url
        self.client.login(username='su', password='pwd')
        url = reverse('list_members', args=('foo.example.com', 'moderator',))
        response = self.client.post(url, {'email': 'newmod@example.com'})
        self.assertRedirects(response, url)

    def tearDown(self):
        self.foo_list.delete()
        self.su.delete()
        self.domain.delete()

    def test_new_moderator_added(self):
        mods_emails = [mod.email for mod in self.foo_list.moderators]
        self.assertTrue('newmod@example.com' in mods_emails)


class ListMembersTest(ViewTestCase):
    """Test the list members page."""

    def setUp(self):
        super(ListMembersTest, self).setUp()
        self.domain = self.mm_client.create_domain('example.com')
        self.foo_list = self.domain.create_list('foo')
        self.superuser = User.objects.create_superuser(
            'testsu', 'su@example.com', 'testpass')
        EmailAddress.objects.create(
            user=self.superuser, email=self.superuser.email, verified=True)

    def tearDown(self):
        self.foo_list.delete()
        self.superuser.delete()
        self.domain.delete()

    def test_show_members_page(self):
        self.client.login(username='testsu', password='testpass')
        member_1 = self.foo_list.subscribe(
            'member-1@example.com', pre_verified=True, pre_confirmed=True,
            pre_approved=True)
        member_2 = self.foo_list.subscribe(
            'member-2@example.com', pre_verified=True, pre_confirmed=True,
            pre_approved=True)
        response = self.client.get(reverse(
            'list_members', args=['foo.example.com', 'member']))
        self.assertEqual(response.status_code, 200)
        self.assertEqual(len(response.context['members']), 2)
        self.assertEqual(response.context['members'].paginator.count, 2)
        self.assertContains(response, member_1.email)
        self.assertContains(response, member_2.email)
        self.assertContains(response, '<small>(2)</small>')

    def test_search_members_1(self):
        self.client.login(username='testsu', password='testpass')
        member_1 = self.foo_list.subscribe(
            'member-1@example.com', pre_verified=True, pre_confirmed=True,
            pre_approved=True)
        member_2 = self.foo_list.subscribe(
            'member-2@example.com', pre_verified=True, pre_confirmed=True,
            pre_approved=True)
        response = self.client.get(reverse(
            'list_members', args=['foo.example.com', 'member']),
            {'q': 'example.com'})
        self.assertEqual(response.status_code, 200)
        self.assertEqual(len(response.context['members']), 2)
        self.assertContains(response, member_1.email)
        self.assertContains(response, member_2.email)
        self.assertContains(response, '<small>(2)</small>')
        response = self.client.get(reverse(
            'list_members', args=['foo.example.com', 'member']),
            {'q': 'member-1'})
        self.assertEqual(response.status_code, 200)
        self.assertEqual(len(response.context['members']), 1)
        self.assertContains(response, member_1.email)
        self.assertNotContains(response, member_2.email)
        self.assertContains(response, '<small>(1)</small>')
        response = self.client.get(reverse(
            'list_members', args=['foo.example.com', 'member']),
            {'q': 'not_a_member'})
        self.assertEqual(response.status_code, 200)
        self.assertEqual(len(response.context['members']), 0)
        self.assertNotContains(response, member_1.email)
        self.assertNotContains(response, member_2.email)
        self.assertContains(response, '<small>(0)</small>')
        self.assertEqual(response.context['empty_error'],
                         'No members were found matching the search.')
        self.foo_list.unsubscribe('member-1@example.com')
        self.foo_list.unsubscribe('member-2@example.com')
        response = self.client.get(reverse(
            'list_members', args=['foo.example.com', 'member']))
        self.assertEqual(response.status_code, 200)
        self.assertEqual(len(response.context['members']), 0)
        self.assertEqual(response.context['empty_error'],
                         'List has no members')
        self.assertContains(response, '<small>(0)</small>')


class ListNonMembersTest(ViewTestCase):
    """Test list non-members"""

    def setUp(self):
        super().setUp()
        self.domain = self.mm_client.create_domain('example.com')
        self.foo_list = self.domain.create_list('foo')
        self.superuser = User.objects.create_superuser(
            'testsu', 'su@example.com', 'testpass')
        EmailAddress.objects.create(
            user=self.superuser, email=self.superuser.email, verified=True)

    def tearDown(self):
        self.foo_list.delete()
        self.superuser.delete()
        self.domain.delete()

    def test_show_nonmembers(self):
        self.client.login(username='testsu', password='testpass')
        self.foo_list.add_role(
            address='nonmember-1@example.com', role='nonmember')
        self.foo_list.add_role(
            address='nonmember-2@example.com', role='nonmember')
        self.foo_list.add_role(
            address='nonmember-3@example.com', role='nonmember')
        response = self.client.get(reverse(
            'list_members', args=['foo.example.com', 'nonmember']))
        self.assertEqual(response.status_code, 200)
        self.assertEqual(len(response.context['members']), 3)
        self.assertEqual(response.context['members'].paginator.count, 3)
        self.assertContains(response, 'nonmember-1@example.com')
        self.assertContains(response, 'nonmember-2@example.com')
        self.assertContains(response, 'nonmember-3@example.com')
        self.assertContains(response, '<small>(3)</small>')

    def test_search_nonmembers(self):
        self.client.login(username='testsu', password='testpass')
        self.foo_list.add_role(
            address='nonmember-1@example.com', role='nonmember')
        self.foo_list.add_role(
            address='nonmember-2@example.com', role='nonmember')
        self.foo_list.add_role(
            address='nonmember-3@example.com', role='nonmember')
        response = self.client.get(reverse(
            'list_members', args=['foo.example.com', 'nonmember']),
            {'q': '2@example.com'})
        self.assertEqual(response.status_code, 200)
        self.assertEqual(len(response.context['members']), 1)
        self.assertContains(response, 'nonmember-2@example.com')
        self.assertContains(response, '<small>(1)</small>')
        response = self.client.get(reverse(
            'list_members', args=['foo.example.com', 'nonmember']),
            {'q': 'random_query'})
        self.assertEqual(response.status_code, 200)
        self.assertEqual(len(response.context['members']), 0)
        self.assertEqual(response.context['empty_error'],
                         'No nonmembers were found matching the search.')
        self.assertContains(response, '<small>(0)</small>')

    def test_add_nonmember(self):
        self.client.login(username='testsu', password='testpass')
        # Check that there aren't any nonmembers.
        self.assertEqual(len(self.foo_list.nonmembers), 0)
        # Add a new nonmember.
        url = reverse('list_members', args=('foo.example.com', 'nonmember',))
        response = self.client.post(url,
                                    {'email': 'new.nonmember@example.com'})
        self.assertRedirects(response, url)
        response = self.client.get(reverse(
            'list_members', args=['foo.example.com', 'nonmember']))
        self.assertEqual(response.status_code, 200)
        self.assertEqual(len(response.context['members']), 1)
        self.assertEqual(response.context['members'][0].email,
                         'new.nonmember@example.com')

    def test_nonmember_delete(self):
        self.client.login(username='testsu', password='testpass')
        self.foo_list.add_role(
            address='nonmember-1@example.com', role='nonmember')
        self.foo_list.add_role(
            address='nonmember-2@example.com', role='nonmember')
        self.foo_list.add_role(
            address='nonmember-3@example.com', role='nonmember')
        nonmember_emails = [nm.email for nm in self.foo_list.nonmembers]
        self.assertEqual(
            sorted(nonmember_emails),
            ['nonmember-1@example.com',
             'nonmember-2@example.com',
             'nonmember-3@example.com'])
        response = self.client.post(
            reverse('remove_role', args=(
                'foo.example.com', 'nonmember', 'nonmember-1@example.com')))
        self.assertEqual(response.status_code, 302)
        self.assertHasSuccessMessage(response)
        nonmember_emails = [nm.email for nm in self.foo_list.nonmembers]
        self.assertEqual(len(nonmember_emails), 2)
        self.assertFalse('nonmember-1@example.com' in nonmember_emails)



# -*- coding: utf-8 -*-
# Copyright (C) 2018-2021 by the Free Software Foundation, Inc.
#
# This file is part of Postorius.
#
# Postorius is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option)
# any later version.
#
# Postorius is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along with
# Postorius.  If not, see <http://www.gnu.org/licenses/>.
#

from django.contrib.auth.models import User
from django.urls import reverse

from allauth.account.models import EmailAddress

from postorius.tests.utils import ViewTestCase


class TestSystemInformationPage(ViewTestCase):

    def setUp(self):
        super().setUp()
        self.superuser = User.objects.create_superuser(
            'testsu', 'su@example.com', 'testpass')
        EmailAddress.objects.create(
            user=self.superuser, email='su@example.com', verified=True)

    def test_system_info(self):
        response = self.client.get(reverse('system_information'))
        # Logged-out users shouldn't be able to get this information.
        self.assertEqual(response.status_code, 302)
        # Superuser should have access to this information.
        self.assertTrue(self.client.login(
            username='testsu', password='testpass'))
        response = self.client.get(reverse('system_information'))
        self.assertEqual(response.status_code, 200)
        self.assertIn(b'Mailman Core API Version', response.content)
        self.assertIn(b'Mailman Core Version', response.content)
